#coding:utf-8
'''
Created on 2016年6月25日

@author: Leo
'''

# Main ToasterBoxDemo

import wx
import wx.lib.scrolledpanel as scrolled

import os
import sys

try:
    dirName = os.path.dirname(os.path.abspath(__file__))
except:
    dirName = os.path.dirname(os.path.abspath(sys.argv[0]))

sys.path.append(os.path.split(dirName)[0])

try:
    from agw import toasterbox as TB
    bitmapDir = "bitmaps/"
except ImportError: # if it's not there locally, try the wxPython lib.
    import wx.lib.agw.toasterbox as TB
    bitmapDir = "agw/bitmaps/"

# In case of TB_COMPLEX style, create a panel that contains an image, some
# text, an hyperlink and a ticker.

import wx.lib.agw.hyperlink as hyperlink
from wx.lib.ticker import Ticker

# ------------------------------------------------------------------------------ #
# Class ToasterBoxDemo
#    This class implements the demo for toasterbox control. try to change the
#    style using the radiobox in the upper section of the frame, and see how
#    ToasterBox acts.
# ------------------------------------------------------------------------------ #

class ToasterBoxDemo(scrolled.ScrolledPanel):

    def __init__(self, parent, log):

        scrolled.ScrolledPanel.__init__(self, parent)

        self.log = log
        
        mainSz = wx.BoxSizer(wx.VERTICAL)

        horSz0 = wx.BoxSizer(wx.HORIZONTAL)
        mainSz.Add((0, 3))
        mainSz.Add(horSz0, 1, wx.EXPAND | wx.BOTTOM, 7)

        sampleList = [" ToasterBox TB_SIMPLE ", " ToasterBox TB_COMPLEX "]
        rb = wx.RadioBox(self, -1, "ToasterBox Style", wx.DefaultPosition,
                         wx.DefaultSize, sampleList, 2, wx.RA_SPECIFY_COLS)

        horSz0.Add(rb, 1, 0, 5)        
        rb.SetToolTip(wx.ToolTip("Choose the ToasterBox style"))

        self.radiochoice = rb
                
        horSz1 = wx.BoxSizer(wx.HORIZONTAL)
        mainSz.Add(horSz1, 1, wx.EXPAND | wx.ALL, 5)
        
        statTxt1 = wx.StaticText(self, -1, "Popup position x/y")
        horSz1.Add(statTxt1, 3)
        txtCtrl1 = wx.TextCtrl(self, -1, "500")
        horSz1.Add(txtCtrl1, 1)
        txtCtrl1b = wx.TextCtrl(self, -1, "500")
        horSz1.Add(txtCtrl1b, 1)

        self.posx = txtCtrl1
        self.posy = txtCtrl1b

        horSz2 = wx.BoxSizer(wx.HORIZONTAL)
        mainSz.Add(horSz2, 1, wx.EXPAND | wx.ALL, 5)
        
        statTxt2 = wx.StaticText(self, -1, "Popup size x/y")
        horSz2.Add(statTxt2, 3)
        txtCtrl2 = wx.TextCtrl(self, -1, "210")
        horSz2.Add(txtCtrl2, 1)
        txtCtrl3 = wx.TextCtrl(self, -1, "130")
        horSz2.Add(txtCtrl3, 1)

        self.sizex = txtCtrl2
        self.sizey = txtCtrl3
        
        horSz3 = wx.BoxSizer(wx.HORIZONTAL)
        mainSz.Add(horSz3, 1, wx.EXPAND | wx.ALL, 5)
        
        statTxt3 = wx.StaticText(self, -1, "Popup linger")
        horSz3.Add(statTxt3, 3)
        txtCtrl4 = wx.TextCtrl(self, -1, "4000")
        helpstr = "How long the popup will stay\naround after it is launched"
        txtCtrl4.SetToolTip(wx.ToolTip(helpstr))
        horSz3.Add(txtCtrl4, 1)

        self.linger = txtCtrl4

        horSz3b = wx.BoxSizer(wx.HORIZONTAL)
        mainSz.Add(horSz3b, 1, wx.EXPAND | wx.ALL, 5)
        statTxt3b = wx.StaticText(self, -1, "Popup scroll speed")
        horSz3b.Add(statTxt3b, 3)
        txtCtrl4b = wx.TextCtrl(self, -1, "8")
        helpstr = "How long it takes the window to \"fade\" in and out"
        txtCtrl4b.SetToolTip(wx.ToolTip(helpstr))
        horSz3b.Add(txtCtrl4b, 2)

        self.scrollspeed = txtCtrl4b

        horSz3c = wx.BoxSizer(wx.HORIZONTAL)
        mainSz.Add(horSz3c, 1, wx.EXPAND | wx.ALL, 5)  
        statTxt3c = wx.StaticText(self, -1, "Popup background picture")
        horSz3c.Add(statTxt3c, 3)
        txtCtrl4c = wx.FilePickerCtrl(self, -1, style=wx.FLP_USE_TEXTCTRL|wx.FLP_OPEN)
        horSz3c.Add(txtCtrl4c, 2)

        self.backimage = txtCtrl4c

        popupText1 = "Hello from wxPython! This is another (probably) useful class. " \
                     "written by Andrea Gavana @ 8 September 2005."
        popupText2 = "I don't know what to write in this message. If you like this " \
                     "class, please let me know!."
        
        horSz4 = wx.BoxSizer(wx.HORIZONTAL)
        mainSz.Add(horSz4, 1, wx.EXPAND | wx.ALL, 5)
        statTxt4 = wx.StaticText(self, -1, "Popup text")
        horSz4.Add(statTxt4, 1)
        txtCtrl5 = wx.TextCtrl(self, -1, popupText1)
        horSz4.Add(txtCtrl5, 2)

        self.showntext = txtCtrl5
        self.popupText1 = popupText1
        self.popupText2 = popupText2
        self.counter = 0

        horSz5 = wx.BoxSizer(wx.HORIZONTAL)
        mainSz.Add(horSz5, 1, wx.EXPAND | wx.ALL, 5)
        self.colButton1 = wx.Button(self, -1, "Set BG Colour")
        self.colButton1.SetToolTip(wx.ToolTip("Set the ToasterBox background colour"))
        self.colButton1.Bind(wx.EVT_BUTTON, self.SetColours)
        horSz5.Add(self.colButton1, 1, 0, 5)
        self.colButton2 = wx.Button(self, -1, "Set FG Colour")
        self.colButton2.SetToolTip(wx.ToolTip("Set the ToasterBox text colour"))
        self.colButton2.Bind(wx.EVT_BUTTON, self.SetColours2)
        horSz5.Add(self.colButton2, 1, 0, 5)

        horSz6 = wx.BoxSizer(wx.HORIZONTAL)
        mainSz.Add(horSz6, 1, wx.EXPAND | wx.ALL, 5)
        statTxt6 = wx.StaticText(self, -1, "Popup text font")
        horSz6.Add(statTxt6, 1, 0, 5)
        fontbutton = wx.Button(self, -1, "Select font")
        horSz6.Add(fontbutton, 1, 0, 5)

        horSz7 = wx.BoxSizer(wx.HORIZONTAL)
        mainSz.Add(horSz7, 1, wx.EXPAND | wx.ALL, 5)
        self.checkcaption = wx.CheckBox(self, -1, "Show with caption")
        horSz7.Add(self.checkcaption, 1, 0, 5)
        self.captiontext = wx.TextCtrl(self, -1, "ToasterBox title!")
        horSz7.Add(self.captiontext, 1, 0, 5)
        self.captiontext.Enable(False)
        self.checkcaption.Bind(wx.EVT_CHECKBOX, self.OnCheckCaption)

        horSz8 = wx.BoxSizer(wx.VERTICAL)
        mainSz.Add(horSz8, 1, wx.EXPAND | wx.ALL, 5)
        self.radiotime = wx.RadioButton(self, -1, "Hide by time", style=wx.RB_GROUP)
        horSz8.Add(self.radiotime, 1, 0, 5)
        self.radioclick = wx.RadioButton(self, -1, "Hide by click")
        horSz8.Add(self.radioclick, 1, 0, 5)
        
        horSz9 = wx.BoxSizer(wx.HORIZONTAL)
        mainSz.Add(horSz9, 1, wx.EXPAND | wx.ALL, 5)
        goButton = wx.Button(self, -1, "Show ToasterBox!")
        goButton.SetToolTip(wx.ToolTip("Launch ToasterBox. You can click more than once!"))
        horSz9.Add((1,0), 1)
        horSz9.Add(goButton, 2, 0, 5)
        horSz9.Add((1,0), 1)

        self.colButton1.SetBackgroundColour(wx.WHITE)
        self.colButton2.SetBackgroundColour(wx.BLACK)
        self.colButton2.SetForegroundColour(wx.WHITE)

        goButton.Bind(wx.EVT_BUTTON, self.ButtonDown)
        fontbutton.Bind(wx.EVT_BUTTON, self.OnSelectFont)
        rb.Bind(wx.EVT_RADIOBOX, self.OnRadioBox)
        
        self.curFont = self.GetFont()
        
        self.SetAutoLayout(True)
        self.SetSizer(mainSz)
        self.Fit()
        self.SetupScrolling()


    def SetColours(self, event):
        
        cd = wx.ColourDialog(self)
        cd.ShowModal()
        colBg = cd.GetColourData().GetColour()
        colButton1 = event.GetEventObject()
        colButton1.SetBackgroundColour(colBg)  


    def SetColours2(self, event):
        
        cd = wx.ColourDialog(self)
        cd.ShowModal()
        colFg = cd.GetColourData().GetColour()
        colButton2 = event.GetEventObject()
        colButton2.SetBackgroundColour(colFg)  


    def OnSelectFont(self, event):

        curFont = self.GetFont()
        curClr = wx.BLACK
        data = wx.FontData()
        data.EnableEffects(True)
        data.SetColour(curClr)
        data.SetInitialFont(curFont)
        
        dlg = wx.FontDialog(self, data)
        
        if dlg.ShowModal() == wx.ID_OK:
            data = dlg.GetFontData()
            font = data.GetChosenFont()
            colour = data.GetColour()

            self.curFont = font
            self.curClr = colour

        dlg.Destroy()        
    

    def OnRadioBox(self, event):

        mainsizer = self.GetSizer()
        
        if event.GetInt() == 0:
            self.linger.SetValue("4000")
            self.scrollspeed.SetValue("8")
            
            for ii in range(5, 10):
                mainsizer.Show(ii, True)
                        
        else:
            for ii in range(5, 10):
                mainsizer.Show(ii, False)

            self.linger.SetValue("10000")
            self.scrollspeed.SetValue("20")

        mainsizer.Layout()
        self.Refresh()
        
        event.Skip()
        

    def OnCheckCaption(self, event):

        if self.checkcaption.GetValue():
            self.captiontext.Enable(True)
            self.sizex.SetValue("250")
            self.sizey.SetValue("200")
        else:
            self.captiontext.Enable(False)
            self.sizex.SetValue("250")
            self.sizey.SetValue("200")

        self.sizex.Refresh()
        self.sizey.Refresh()
        

    def ButtonDown(self, event):

        demochoice = self.radiochoice.GetSelection()

        if self.checkcaption.GetValue():
            txts = self.captiontext.GetValue().strip()
            windowstyle = TB.TB_CAPTION
        else:
            windowstyle = TB.TB_DEFAULT_STYLE
        
        if demochoice == 1:
            tbstyle = TB.TB_COMPLEX
        else:
            tbstyle = TB.TB_SIMPLE

        if self.radioclick.GetValue():
            closingstyle = TB.TB_ONCLICK
        else:
            closingstyle = TB.TB_ONTIME
            
        tb = TB.ToasterBox(self, tbstyle, windowstyle, closingstyle,
                           scrollType=TB.TB_SCR_TYPE_FADE
                           )

        if windowstyle == TB.TB_CAPTION:
            tb.SetTitle(txts)

        sizex = int(self.sizex.GetValue())
        sizey = int(self.sizey.GetValue())
        tb.SetPopupSize((sizex, sizey))

        posx = int(self.posx.GetValue())
        posy = int(self.posy.GetValue())
        tb.SetPopupPosition((posx, posy))
        
        tb.SetPopupPauseTime(int(self.linger.GetValue()))
        tb.SetPopupScrollSpeed(int(self.scrollspeed.GetValue()))

        if demochoice == 0:    # Simple Demo:

            self.RunSimpleDemo(tb)

        else:

            self.RunComplexDemo(tb)            
        
        tb.Play()


    def RunSimpleDemo(self, tb):

        tb.SetPopupBackgroundColour(self.colButton1.GetBackgroundColour())
        tb.SetPopupTextColour(self.colButton2.GetBackgroundColour())
        bmp = self.backimage.GetPath()
        dummybmp = wx.NullBitmap
        
        if os.path.isfile(bmp):
            dummybmp = wx.Bitmap(bmp)

        if dummybmp.Ok():
            tb.SetPopupBitmap(bmp)
        else:
            tb.SetPopupBitmap()

        txtshown = self.showntext.GetValue()
        if self.counter == 0:
            if txtshown in [self.popupText1, self.popupText2]:
                self.counter = self.counter + 1
                txtshown = self.popupText1
        else:
            if txtshown in [self.popupText1, self.popupText2]:
                self.counter = 0
                txtshown = self.popupText2             
             
        tb.SetPopupText(txtshown)
        tb.SetPopupTextFont(self.curFont)
            

    def RunComplexDemo(self, tb):

        # This Is The New Call Style: The Call To GetToasterBoxWindow()
        # Is Mandatory, In Order To Create A Custom Parent On ToasterBox.
        
        tbpanel = tb.GetToasterBoxWindow()
        panel = wx.Panel(tbpanel, -1)

        sizer = wx.BoxSizer(wx.VERTICAL)
        horsizer1 = wx.BoxSizer(wx.HORIZONTAL)

        myimage = wx.Bitmap(os.path.join(bitmapDir, "sttfont.png"), wx.BITMAP_TYPE_PNG)
        stbmp = wx.StaticBitmap(panel, -1, myimage)
        horsizer1.Add(stbmp, 0)

        strs = "This Is Another Example Of ToasterBox, A Complex One. This Kind Of"
        strs = strs + " ToasterBox Can Be Created Using The Style TB_COMPLEX."
        sttext = wx.StaticText(panel, -1, strs)
        horsizer1.Add(sttext, 1, wx.EXPAND | wx.LEFT | wx.RIGHT, 5)

        hl = hyperlink.HyperLinkCtrl(panel, -1, "My Home Page",
                                     URL="http://xoomer.alice.it/infinity77/")

        sizer.Add((0,5))        
        sizer.Add(horsizer1, 0, wx.EXPAND)

        horsizer2 = wx.BoxSizer(wx.HORIZONTAL)
        horsizer2.Add((5, 0))
        horsizer2.Add(hl, 0, wx.EXPAND | wx.TOP, 10)
        sizer.Add(horsizer2, 0, wx.EXPAND)

        tk = Ticker(panel)
        tk.SetText("Hello From wxPython!")
        
        horsizer3 = wx.BoxSizer(wx.HORIZONTAL)
        horsizer3.Add((5, 0))
        horsizer3.Add(tk, 1, wx.EXPAND | wx.TOP, 10)
        horsizer3.Add((5,0))
        sizer.Add(horsizer3, 0, wx.EXPAND)
        
        panel.SetSizer(sizer)
        panel.Layout()
        
        tb.AddPanel(panel)
        

class ToasterBox(TB.ToasterBox):
    @staticmethod
    def MakeText(win, info='', tbstyle=TB.TB_SIMPLE, windowstyle=TB.TB_DEFAULT_STYLE,
                 closingstyle=TB.TB_ONTIME, scrollType=TB.TB_SCR_TYPE_FADE):
        
        tbox = ToasterBox(win, tbstyle, windowstyle, closingstyle, scrollType)
        
        if info:
            tbox.SetPopupText(info)
            tbox.SetPopupSize(wx.Size(200, 80))
        else:
            tbox.SetPopupSize(wx.Size(100, 40))

        #tb.SetPopupPosition(win.TopLevelParent.GetPos())
        tbox.CenterOnParent(wx.BOTH)    
        #tbox.SetPopupBackgroundColour("black")
        tbox.SetPopupPauseTime(2000)
        tbox.SetPopupScrollSpeed(8)
        return tbox
    
    def Show(self):
        self.Play()
        


#coding:utf-8
# !/usr/local/python/bin

from __future__ import (print_function, unicode_literals)
from utilityhelper.compatible import *

__author__ = "gongleilei"
__status__ = "Development"

# __all__ = ['set_logger', 'debug', 'info', 'warning', 'error',
#            'critical', 'exception']

import os
import sys
import logging  # @UnusedImport
import logging.handlers

# Color escape string
COLOR_RED = '\033[1;31m'
COLOR_GREEN = '\033[1;32m'
COLOR_YELLOW = '\033[1;33m'
COLOR_BLUE = '\033[1;34m'
COLOR_PURPLE = '\033[1;35m'
COLOR_CYAN = '\033[1;36m'
COLOR_GRAY = '\033[1;37m'
COLOR_WHITE = '\033[1;38m'
COLOR_RESET = '\033[1;0m'

# Define log color
LOG_COLORS = {
    'DEBUG': '%s',
    'INFO': COLOR_GREEN + '%s' + COLOR_RESET,
    'WARNING': COLOR_YELLOW + '%s' + COLOR_RESET,
    'ERROR': COLOR_RED + '%s' + COLOR_RESET,
    'CRITICAL': COLOR_RED + '%s' + COLOR_RESET,
    'EXCEPTION': COLOR_RED + '%s' + COLOR_RESET,
}

# Global logger
g_logger = []

#继承logging模块的日志级别
LEVEL_CRITICAL = logging.CRITICAL
LEVEL_FATAL = logging.FATAL
LEVEL_ERROR = logging.ERROR
LEVEL_WARNING = logging.WARNING
LEVEL_WARN = LEVEL_WARNING
LEVEL_INFO = logging.INFO
LEVEL_DEBUG = logging.DEBUG
LEVEL_NOTSET = logging.NOTSET

HANDLER_TYPE_STREAM = 1
HANDLER_TYPE_FILE   = 2
HANDLER_TYPE_SMTP   = 3
HANDLER_TYPE_WINDOW = 4

class EncodingFormatter(logging.Formatter):

    def __init__(self, fmt, datefmt=None, encoding=None):
        super(EncodingFormatter, self).__init__(fmt, datefmt)
        self.encoding = encoding

    def format(self, record):
        result = logging.Formatter.format(self, record)
        if not PY3:
            if isinstance(result, unicode):
                result = result.encode(self.encoding or 'utf-8')
        return result

class ColoredFormatter(logging.Formatter):
    '''@summary A colorful formatter.
    '''
    def __init__(self, fmt=None, datefmt=None):
        super(ColoredFormatter, self).__init__(fmt, datefmt)

    def format(self, record):
        level_name = record.levelname
        msg = logging.Formatter.format(self, record)
        msg = LOG_COLORS.get(level_name, '%s') % msg
        return msg


class LogManger(object):
    
    @classmethod
    def getLogger(cls, name=None, LowestLevel=logging.WARNING):
        if name is None:
            logger = logging.getLogger()
            if logger not in g_logger:
                logger.setLevel(LowestLevel)
                g_logger.append(logger)
        else:
            logger = logging.getLogger(name)
            if logger not in g_logger:
                logger.setLevel(LowestLevel)
                g_logger.append(logger)
        
        return logger

    @classmethod
    def addHandler(cls, loggers, handlertype, level, fmt, **kwargs):
        if handlertype == HANDLER_TYPE_STREAM:
            LogManger.__add_streamhandler(loggers, level, fmt, **kwargs)
        elif handlertype == HANDLER_TYPE_FILE:
            LogManger.__add_filehandler(loggers, level, fmt, **kwargs)
        elif handlertype == HANDLER_TYPE_SMTP:
            LogManger.__add_mailhandler(loggers, level, fmt, **kwargs)
        elif handlertype == HANDLER_TYPE_WINDOW:
            LogManger.__add_windowhandler(loggers, level, fmt, **kwargs)

    @staticmethod
    def __add_handler(HanderClass, loggers, level, fmt, colorful, **kwargs):
        '''Add a configured handler to the global logger.'''
        global g_logger
        if isinstance(level, str):
            level = getattr(logging, level.upper(), logging.DEBUG)
        handler = HanderClass(**kwargs)
        handler.setLevel(level)
        
        if colorful:
            formatter = ColoredFormatter(fmt)
        else:
            formatter = EncodingFormatter(fmt)
    
        handler.setFormatter(formatter)
        for logger in loggers:
            if g_logger.count(logger) != 0:
                logger.addHandler(handler)
    
        return handler

    @staticmethod
    def __add_streamhandler(loggers, level, fmt):
        '''Add a stream handler to the global logger.'''
        return LogManger.__add_handler(logging.StreamHandler, loggers, level, fmt, False)
    
    @staticmethod
    def __add_filehandler(loggers, level, fmt, filename, mode, backup_count, limit, when):
        '''Add a file handler to the global logger.'''
        kwargs = {}
    
        # If the filename is not set, use the default filename
        if filename is None:
            filename = getattr(sys.modules['__main__'], '__file__', 'log.py')
            filename = os.path.basename(filename.replace('.py', '.log'))
            #path = os.path.abspath(filename)
            #filename = os.path.join('/tmp', filename)
            #filename = os.path.join(path, filename)
        kwargs['filename'] = filename
    
        # Choose the filehandler based on the passed arguments
        if backup_count == 0:  # Use FileHandler
            cls = logging.FileHandler
            kwargs['mode'] = mode
        elif when is None:  # Use RotatingFileHandler
            cls = logging.handlers.RotatingFileHandler
            kwargs['maxBytes'] = limit
            kwargs['backupCount'] = backup_count
            kwargs['mode'] = mode
        else:  # Use TimedRotatingFileHandler
            cls = logging.handlers.TimedRotatingFileHandler
            kwargs['when'] = when
            kwargs['interval'] = limit
            kwargs['backupCount'] = backup_count
    
        return LogManger.__add_handler(cls, loggers, level, fmt, False, **kwargs)

    @staticmethod
    def __add_windowhandler(loggers, level, fmt, window):
        '''Add a window handler to the global logger.'''
        kwargs = {}
        kwargs['stream'] = window
        return LogManger.__add_handler(logging.StreamHandler, loggers, level, fmt, False, **kwargs)

    @staticmethod
    def __add_mailhandler(loggers, level, fmt, 
                        mailhost, fromaddr, toaddrs, subject,
                        credentials=None, secure=None):
        kwargs = {}
        kwargs['mailhost'] = mailhost
        kwargs['fromaddr'] = fromaddr
        kwargs['toaddrs'] = toaddrs
        kwargs['subject'] = subject
        kwargs['credentials'] = credentials
        kwargs['secure'] = secure
        
        return LogManger.__add_handler(logging.handlers.SMTPHandler, loggers, level, fmt, True, **kwargs)

if __name__ == "__main__":
    __fmt = '【%(levelname)s】 - %(asctime)s - %(name)s -%(thread)d -%(threadName)s - %(message)s'
    _g_logger = LogManger.getLogger(None, LEVEL_DEBUG)
    LogManger.addHandler((_g_logger,), HANDLER_TYPE_STREAM, LEVEL_DEBUG, __fmt)
    _g_logger.debug("logdebug={}".format('message'))



#! python
# Python Serial Port Extension for Win32, Linux, BSD, Jython
# see __init__.py
#
# (C) 2001-2010 Chris Liechti <cliechti@gmx.net>
# this is distributed under a free software license, see license.txt

# compatibility for older Python < 2.6
try:
    bytes
    bytearray
except (NameError, AttributeError):
    # Python older than 2.6 do not have these types. Like for Python 2.6 they
    # should behave like str. For Python older than 3.0 we want to work with
    # strings anyway, only later versions have a true bytes type.
    bytes = str
    # bytearray is a mutable type that is easily turned into an instance of
    # bytes
    class bytearray(list):
        # for bytes(bytearray()) usage
        def __str__(self): return ''.join(self)
        def __repr__(self): return 'bytearray(%r)' % ''.join(self)
        # append automatically converts integers to characters
        def append(self, item):
            if isinstance(item, str):
                list.append(self, item)
            else:
                list.append(self, chr(item))
        # +=
        def __iadd__(self, other):
            for byte in other:
                self.append(byte)
            return self

        def __getslice__(self, i, j):
            return bytearray(list.__getslice__(self, i, j))

        def __getitem__(self, item):
            if isinstance(item, slice):
                return bytearray(list.__getitem__(self, item))
            else:
                return ord(list.__getitem__(self, item))

        def __eq__(self, other):
            if isinstance(other, str):
                other = bytearray(other)
            return list.__eq__(self, other)

# ``memoryview`` was introduced in Python 2.7 and ``bytes(some_memoryview)``
# isn't returning the contents (very unfortunate). Therefore we need special
# cases and test for it. Ensure that there is a ``memoryview`` object for older
# Python versions. This is easier than making every test dependent on its
# existence.
try:
    memoryview
except (NameError, AttributeError):
    # implementation does not matter as we do not realy use it.
    # it just must not inherit from something else we might care for.
    class memoryview:
        pass


# all Python versions prior 3.x convert ``str([17])`` to '[17]' instead of '\x11'
# so a simple ``bytes(sequence)`` doesn't work for all versions
def to_bytes(seq):
    """convert a sequence to a bytes type"""
    if isinstance(seq, bytes):
        return seq
    elif isinstance(seq, bytearray):
        return bytes(seq)
    elif isinstance(seq, memoryview):
        return seq.tobytes()
    else:
        b = bytearray()
        for item in seq:
            b.append(item)  # this one handles int and str for our emulation and ints for Python 3.x
        return bytes(b)

# create control bytes
XON  = to_bytes([17])
XOFF = to_bytes([19])

CR = to_bytes([13])
LF = to_bytes([10])


PARITY_NONE, PARITY_EVEN, PARITY_ODD, PARITY_MARK, PARITY_SPACE = 'N', 'E', 'O', 'M', 'S'
STOPBITS_ONE, STOPBITS_ONE_POINT_FIVE, STOPBITS_TWO = (1, 1.5, 2)
FIVEBITS, SIXBITS, SEVENBITS, EIGHTBITS = (5, 6, 7, 8)

PARITY_NAMES = {
    PARITY_NONE:  'None',
    PARITY_EVEN:  'Even',
    PARITY_ODD:   'Odd',
    PARITY_MARK:  'Mark',
    PARITY_SPACE: 'Space',
}


class SerialException(IOError):
    """Base class for serial port related exceptions."""


class SerialTimeoutException(SerialException):
    """Write timeouts give an exception"""


writeTimeoutError = SerialTimeoutException('Write timeout')
portNotOpenError = SerialException('Attempting to use a port that is not open')


class FileLike(object):
    """An abstract file like class.

    This class implements readline and readlines based on read and
    writelines based on write.
    This class is used to provide the above functions for to Serial
    port objects.

    Note that when the serial port was opened with _NO_ timeout that
    readline blocks until it sees a newline (or the specified size is
    reached) and that readlines would never return and therefore
    refuses to work (it raises an exception in this case)!
    """

    def __init__(self):
        self.closed = True

    def close(self):
        self.closed = True

    # so that ports are closed when objects are discarded
    def __del__(self):
        """Destructor.  Calls close()."""
        # The try/except block is in case this is called at program
        # exit time, when it's possible that globals have already been
        # deleted, and then the close() call might fail.  Since
        # there's nothing we can do about such failures and they annoy
        # the end users, we suppress the traceback.
        try:
            self.close()
        except:
            pass

    def writelines(self, sequence):
        for line in sequence:
            self.write(line)

    def flush(self):
        """flush of file like objects"""
        pass

    # iterator for e.g. "for line in Serial(0): ..." usage
    def __next__(self):
        line = self.readline()
        if not line: raise StopIteration
        return line

    def __iter__(self):
        return self

    def readline(self, size=None, eol=LF):
        """read a line which is terminated with end-of-line (eol) character
        ('\n' by default) or until timeout."""
        leneol = len(eol)
        line = bytearray()
        while True:
            c = self.read(1)
            if c:
                line += c
                if line[-leneol:] == eol:
                    break
                if size is not None and len(line) >= size:
                    break
            else:
                break
        return bytes(line)

    def readlines(self, sizehint=None, eol=LF):
        """read a list of lines, until timeout.
        sizehint is ignored."""
        if self.timeout is None:
            raise ValueError("Serial port MUST have enabled timeout for this function!")
        leneol = len(eol)
        lines = []
        while True:
            line = self.readline(eol=eol)
            if line:
                lines.append(line)
                if line[-leneol:] != eol:    # was the line received with a timeout?
                    break
            else:
                break
        return lines

    def xreadlines(self, sizehint=None):
        """Read lines, implemented as generator. It will raise StopIteration on
        timeout (empty read). sizehint is ignored."""
        while True:
            line = self.readline()
            if not line: break
            yield line

    # other functions of file-likes - not used by pySerial

    #~ readinto(b)

    def seek(self, pos, whence=0):
        raise IOError("file is not seekable")

    def tell(self):
        raise IOError("file is not seekable")

    def truncate(self, n=None):
        raise IOError("file is not seekable")

    def isatty(self):
        return False


class SerialBase(object):
    """Serial port base class. Provides __init__ function and properties to
       get/set port settings."""

    # default values, may be overridden in subclasses that do not support all values
    BAUDRATES = (50, 75, 110, 134, 150, 200, 300, 600, 1200, 1800, 2400, 4800,
                 9600, 19200, 38400, 57600, 115200, 230400, 460800, 500000,
                 576000, 921600, 1000000, 1152000, 1500000, 2000000, 2500000,
                 3000000, 3500000, 4000000)
    BYTESIZES = (FIVEBITS, SIXBITS, SEVENBITS, EIGHTBITS)
    PARITIES  = (PARITY_NONE, PARITY_EVEN, PARITY_ODD, PARITY_MARK, PARITY_SPACE)
    STOPBITS  = (STOPBITS_ONE, STOPBITS_ONE_POINT_FIVE, STOPBITS_TWO)

    def __init__(self,
                 port = None,           # number of device, numbering starts at
                                        # zero. if everything fails, the user
                                        # can specify a device string, note
                                        # that this isn't portable anymore
                                        # port will be opened if one is specified
                 baudrate=9600,         # baud rate
                 bytesize=EIGHTBITS,    # number of data bits
                 parity=PARITY_NONE,    # enable parity checking
                 stopbits=STOPBITS_ONE, # number of stop bits
                 timeout=None,          # set a timeout value, None to wait forever
                 xonxoff=False,         # enable software flow control
                 rtscts=False,          # enable RTS/CTS flow control
                 writeTimeout=None,     # set a timeout for writes
                 dsrdtr=False,          # None: use rtscts setting, dsrdtr override if True or False
                 interCharTimeout=None  # Inter-character timeout, None to disable
                 ):
        """Initialize comm port object. If a port is given, then the port will be
           opened immediately. Otherwise a Serial port object in closed state
           is returned."""

        self._isOpen   = False
        self._port     = None           # correct value is assigned below through properties
        self._baudrate = None           # correct value is assigned below through properties
        self._bytesize = None           # correct value is assigned below through properties
        self._parity   = None           # correct value is assigned below through properties
        self._stopbits = None           # correct value is assigned below through properties
        self._timeout  = None           # correct value is assigned below through properties
        self._writeTimeout = None       # correct value is assigned below through properties
        self._xonxoff  = None           # correct value is assigned below through properties
        self._rtscts   = None           # correct value is assigned below through properties
        self._dsrdtr   = None           # correct value is assigned below through properties
        self._interCharTimeout = None   # correct value is assigned below through properties

        # assign values using get/set methods using the properties feature
        self.port     = port
        self.baudrate = baudrate
        self.bytesize = bytesize
        self.parity   = parity
        self.stopbits = stopbits
        self.timeout  = timeout
        self.writeTimeout = writeTimeout
        self.xonxoff  = xonxoff
        self.rtscts   = rtscts
        self.dsrdtr   = dsrdtr
        self.interCharTimeout = interCharTimeout

        if port is not None:
            self.open()

    def isOpen(self):
        """Check if the port is opened."""
        return self._isOpen

    #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -

    # TODO: these are not really needed as the is the BAUDRATES etc. attribute...
    # maybe i remove them before the final release...

    def getSupportedBaudrates(self):
        return [(str(b), b) for b in self.BAUDRATES]

    def getSupportedByteSizes(self):
        return [(str(b), b) for b in self.BYTESIZES]

    def getSupportedStopbits(self):
        return [(str(b), b) for b in self.STOPBITS]

    def getSupportedParities(self):
        return [(PARITY_NAMES[b], b) for b in self.PARITIES]

    #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -

    def setPort(self, port):
        """Change the port. The attribute portstr is set to a string that
           contains the name of the port."""

        was_open = self._isOpen
        if was_open: self.close()
        if port is not None:
            if isinstance(port, str):
                self.portstr = port
            else:
                self.portstr = self.makeDeviceName(port)
        else:
            self.portstr = None
        self._port = port
        self.name = self.portstr
        if was_open: self.open()

    def getPort(self):
        """Get the current port setting. The value that was passed on init or using
           setPort() is passed back. See also the attribute portstr which contains
           the name of the port as a string."""
        return self._port

    port = property(getPort, setPort, doc="Port setting")


    def setBaudrate(self, baudrate):
        """Change baud rate. It raises a ValueError if the port is open and the
        baud rate is not possible. If the port is closed, then the value is
        accepted and the exception is raised when the port is opened."""
        try:
            b = int(baudrate)
        except TypeError:
            raise ValueError("Not a valid baudrate: %r" % (baudrate,))
        else:
            if b <= 0:
                raise ValueError("Not a valid baudrate: %r" % (baudrate,))
            self._baudrate = b
            if self._isOpen:  self._reconfigurePort()

    def getBaudrate(self):
        """Get the current baud rate setting."""
        return self._baudrate

    baudrate = property(getBaudrate, setBaudrate, doc="Baud rate setting")


    def setByteSize(self, bytesize):
        """Change byte size."""
        if bytesize not in self.BYTESIZES: raise ValueError("Not a valid byte size: %r" % (bytesize,))
        self._bytesize = bytesize
        if self._isOpen: self._reconfigurePort()

    def getByteSize(self):
        """Get the current byte size setting."""
        return self._bytesize

    bytesize = property(getByteSize, setByteSize, doc="Byte size setting")


    def setParity(self, parity):
        """Change parity setting."""
        if parity not in self.PARITIES: raise ValueError("Not a valid parity: %r" % (parity,))
        self._parity = parity
        if self._isOpen: self._reconfigurePort()

    def getParity(self):
        """Get the current parity setting."""
        return self._parity

    parity = property(getParity, setParity, doc="Parity setting")


    def setStopbits(self, stopbits):
        """Change stop bits size."""
        if stopbits not in self.STOPBITS: raise ValueError("Not a valid stop bit size: %r" % (stopbits,))
        self._stopbits = stopbits
        if self._isOpen: self._reconfigurePort()

    def getStopbits(self):
        """Get the current stop bits setting."""
        return self._stopbits

    stopbits = property(getStopbits, setStopbits, doc="Stop bits setting")


    def setTimeout(self, timeout):
        """Change timeout setting."""
        if timeout is not None:
            try:
                timeout + 1     # test if it's a number, will throw a TypeError if not...
            except TypeError:
                raise ValueError("Not a valid timeout: %r" % (timeout,))
            if timeout < 0: raise ValueError("Not a valid timeout: %r" % (timeout,))
        self._timeout = timeout
        if self._isOpen: self._reconfigurePort()

    def getTimeout(self):
        """Get the current timeout setting."""
        return self._timeout

    timeout = property(getTimeout, setTimeout, doc="Timeout setting for read()")


    def setWriteTimeout(self, timeout):
        """Change timeout setting."""
        if timeout is not None:
            if timeout < 0: raise ValueError("Not a valid timeout: %r" % (timeout,))
            try:
                timeout + 1     #test if it's a number, will throw a TypeError if not...
            except TypeError:
                raise ValueError("Not a valid timeout: %r" % timeout)

        self._writeTimeout = timeout
        if self._isOpen: self._reconfigurePort()

    def getWriteTimeout(self):
        """Get the current timeout setting."""
        return self._writeTimeout

    writeTimeout = property(getWriteTimeout, setWriteTimeout, doc="Timeout setting for write()")


    def setXonXoff(self, xonxoff):
        """Change XON/XOFF setting."""
        self._xonxoff = xonxoff
        if self._isOpen: self._reconfigurePort()

    def getXonXoff(self):
        """Get the current XON/XOFF setting."""
        return self._xonxoff

    xonxoff = property(getXonXoff, setXonXoff, doc="XON/XOFF setting")

    def setRtsCts(self, rtscts):
        """Change RTS/CTS flow control setting."""
        self._rtscts = rtscts
        if self._isOpen: self._reconfigurePort()

    def getRtsCts(self):
        """Get the current RTS/CTS flow control setting."""
        return self._rtscts

    rtscts = property(getRtsCts, setRtsCts, doc="RTS/CTS flow control setting")

    def setDsrDtr(self, dsrdtr=None):
        """Change DsrDtr flow control setting."""
        if dsrdtr is None:
            # if not set, keep backwards compatibility and follow rtscts setting
            self._dsrdtr = self._rtscts
        else:
            # if defined independently, follow its value
            self._dsrdtr = dsrdtr
        if self._isOpen: self._reconfigurePort()

    def getDsrDtr(self):
        """Get the current DSR/DTR flow control setting."""
        return self._dsrdtr

    dsrdtr = property(getDsrDtr, setDsrDtr, "DSR/DTR flow control setting")

    def setInterCharTimeout(self, interCharTimeout):
        """Change inter-character timeout setting."""
        if interCharTimeout is not None:
            if interCharTimeout < 0: raise ValueError("Not a valid timeout: %r" % interCharTimeout)
            try:
                interCharTimeout + 1     # test if it's a number, will throw a TypeError if not...
            except TypeError:
                raise ValueError("Not a valid timeout: %r" % interCharTimeout)

        self._interCharTimeout = interCharTimeout
        if self._isOpen: self._reconfigurePort()

    def getInterCharTimeout(self):
        """Get the current inter-character timeout setting."""
        return self._interCharTimeout

    interCharTimeout = property(getInterCharTimeout, setInterCharTimeout, doc="Inter-character timeout setting for read()")

    #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -

    _SETTINGS = ('baudrate', 'bytesize', 'parity', 'stopbits', 'xonxoff',
            'dsrdtr', 'rtscts', 'timeout', 'writeTimeout', 'interCharTimeout')

    def getSettingsDict(self):
        """Get current port settings as a dictionary. For use with
        applySettingsDict"""
        return dict([(key, getattr(self, '_'+key)) for key in self._SETTINGS])

    def applySettingsDict(self, d):
        """apply stored settings from a dictionary returned from
        getSettingsDict. it's allowed to delete keys from the dictionary. these
        values will simply left unchanged."""
        for key in self._SETTINGS:
            if d[key] != getattr(self, '_'+key):   # check against internal "_" value
                setattr(self, key, d[key])          # set non "_" value to use properties write function

    #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -

    def __repr__(self):
        """String representation of the current port settings and its state."""
        return "%s<id=0x%x, open=%s>(port=%r, baudrate=%r, bytesize=%r, parity=%r, stopbits=%r, timeout=%r, xonxoff=%r, rtscts=%r, dsrdtr=%r)" % (
            self.__class__.__name__,
            id(self),
            self._isOpen,
            self.portstr,
            self.baudrate,
            self.bytesize,
            self.parity,
            self.stopbits,
            self.timeout,
            self.xonxoff,
            self.rtscts,
            self.dsrdtr,
        )


    #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
    # compatibility with io library

    def readable(self): return True
    def writable(self): return True
    def seekable(self): return False
    def readinto(self, b):
        data = self.read(len(b))
        n = len(data)
        try:
            b[:n] = data
        except TypeError as err:
            import array
            if not isinstance(b, array.array):
                raise err
            b[:n] = array.array('b', data)
        return n


if __name__ == '__main__':
    import sys
    s = SerialBase()
    sys.stdout.write('port name:  %s\n' % s.portstr)
    sys.stdout.write('baud rates: %s\n' % s.getSupportedBaudrates())
    sys.stdout.write('byte sizes: %s\n' % s.getSupportedByteSizes())
    sys.stdout.write('parities:   %s\n' % s.getSupportedParities())
    sys.stdout.write('stop bits:  %s\n' % s.getSupportedStopbits())
    sys.stdout.write('%s\n' % s)



#coding:utf-8
from __future__ import (print_function, unicode_literals)
from Crypto.Cipher import DES3, DES

DEBUG_DES = 1

class MyDES(object):
    
    skey = ''
    mkey = ''
       
    @classmethod 
    def encrypt(self, data):
        '''3DES ECB mode.
            
        '''
        D = data   

        '''在明文M后附加0x80，然后在右端填充最少的0x00，使得填充后消息M = (M||80||00||00||…||00)的长度为8的整数倍'''
        D += b'\x80'
        if len(D)%8:
            D += b'\x00' * (8 - len(D)%8)

        cipher = DES3.new(self.skey, DES3.MODE_ECB)
        EData = cipher.encrypt(D)
        return EData
    
    @classmethod
    def decrypt(self, EData):

        cipher = DES3.new(self.skey, DES3.MODE_ECB)
        D = cipher.decrypt(EData)
        return D
    
    @classmethod
    def CalcMAC(self, data):
        D = data + b'\x80'
        if len(D)%8:
            D += b'\x00' * (8 - len(D)%8)
        
        #CBC模式加密
        Ecipher = DES.new(self.mkey[:8], DES.MODE_CBC, IV=b'\x00'*8)
        C = Ecipher.encrypt(D)
        
        #使用最后一块数据计算消息认证码
        Cn = C[-8:]
        Dcipher = DES.new(self.mkey[8:], DES.MODE_ECB)
        Dn = Dcipher.decrypt(Cn)
        
        Ecipher = DES.new(self.mkey[:8], DES.MODE_ECB)
        Mac = Ecipher.encrypt(Dn)

        return Mac


#coding:utf-8
from __future__ import (print_function, unicode_literals)
import time
try:
    import wx
except:
    try:
        import PyQt5
    except:
        pass

def wrapper_call_after(func):
    def _wrapper(*args, **kwargs):
        return wx.CallAfter(func, *args, **kwargs)
    return _wrapper

#TODO
def wr_prog_exec_time_out(elapse=None):

    def wrapper_prog_exec_time(func):
        """
        包含了其他程序使用CPU的时间，是程序开始到程序结束的运行时间。
        :param func:
        :return:
        """
        if elapse is None:
            elapse = []
        def _wrapper(*args, **kwargs):
            start = time.time()
            ret = func(*args, **kwargs)
            end = time.time()
            return end - start, ret
        return _wrapper
    return wrapper_prog_exec_time

def wrapper_prog_exec_time(func):
    """
    包含了其他程序使用CPU的时间，是程序开始到程序结束的运行时间。
    :param func:
    :return:
    """

    def _wrapper(*args, **kwargs):
        start = time.time()
        ret = func(*args, **kwargs)
        end = time.time()
        return end - start
    return _wrapper


def wrapper_cpu_exec_time(func):
    """
    只计算了程序运行的CPU时间，相对准确
    :param func:
    :return:
    """
    def wrapper(*args, **kwargs):
        start = time.clock()  
        func(*args, **kwargs)
        end =time.clock()
        print('used:', end - start)
        return end - start
    return wrapper

def wrapper_round_n_float(radix):
    def _wrapper(fun):
        def return_fun(*args, **kwargs):
            ret = fun(*args, **kwargs)
            return round(ret, radix)
        return return_fun
    return _wrapper

def wrapper_print_args(func):
    def _wrapper(*args, **kwargs):
        print("Arguments were: %s, %s" % (args, kwargs))
        ret = func(*args, **kwargs)
        return ret
    return _wrapper


def singleton(cls):
    import functools
    instance={}
    @functools.wraps(cls)
    def _singleton(*args, **kw):
        if cls not in instance:
            instance[cls]=cls(*args, **kw)
        return instance[cls]
    return _singleton

if __name__ == "__main__":
    pass


#coding:utf-8
'''
01, 20150626, add function <toDerFmt> to support certificate file encoding format base64 to der.
'''
from __future__ import (print_function, unicode_literals)
import base64

from Crypto.Hash import SHA
from Crypto.PublicKey import RSA
from Crypto.Util import number
from utilityhelper.common.util.CertUil import Codeutil
from pyasn1.codec.der import decoder, encoder
from pyasn1_modules import rfc2459


class ValueOnlyBitStringEncoder(encoder.encoder.BitStringEncoder):
    # These methods just do not encode tag and length fields of TLV
    def encodeTag(self, *args): return ''
    def encodeLength(self, *args): return ''
    def encodeValue(*args):
        substrate, isConstructed = encoder.encoder.BitStringEncoder.encodeValue(*args)
        # OCSP-specific hack follows: cut off the "unused bit count"
        # encoded bit-string value.
        return substrate[1:], isConstructed

    def __call__(self, bitStringValue):
        return self.encode(None, bitStringValue, defMode=1, maxChunkSize=0)

valueOnlyBitStringEncoder = ValueOnlyBitStringEncoder()

id = {
    rfc2459.id_at_commonName : 'commonName',                #2.5.4.3  通用名(CN)，设备名称、域名或IP、厂商名称等
    rfc2459.id_at_countryName : 'countryName',              #2.5.4.6  国家(C)  
    rfc2459.id_at_organizationName : 'organizationName',    #2.5.4.10 单位(O)，颁发机构
    rfc2459.id_at_organizationalUnitName : 'organizationalUnitName',  #2.5.4.11 机构名称(OU)，证书类型(OU)           
    rfc2459.id_at_givenName : 'givenName',                  #2.5.4.42
    rfc2459.id_at_stateOrProvinceName: "stateOrProvinceName",
    rfc2459.id_at_localityName : 'localityName',
    
    # Algorithm OIDs and parameter structures
    rfc2459.pkcs_1 : "pkcs_1",                                  #('1.2.840.113549.1.1')
    rfc2459.rsaEncryption: "rsaEncryption",                     #= univ.ObjectIdentifier('1.2.840.113549.1.1.1')
    rfc2459.md2WithRSAEncryption : "md2WithRSAEncryption",      #= univ.ObjectIdentifier('1.2.840.113549.1.1.2')
    rfc2459.md5WithRSAEncryption : "md5WithRSAEncryption",      #= univ.ObjectIdentifier('1.2.840.113549.1.1.4')
    rfc2459.sha1WithRSAEncryption : "sha1WithRSAEncryption",    #= univ.ObjectIdentifier('1.2.840.113549.1.1.5')
    rfc2459.id_dsa_with_sha1 : "id_dsa_with_sha1",               #= univ.ObjectIdentifier('1.2.840.10040.4.3')    
}
        
class Cert(object):
    
    def __init__(self, cert, isfile=False):
        if isfile:
            f = open(cert, "rb")
            cert = f.read()
            f.close()
        certType = rfc2459.Certificate()
        self.cert, rest = decoder.decode(cert, asn1Spec=certType)
        
#         print "===========GetSubjectDN=============="
#         self.GetSubjectDN()
#         print "===========GetSubjectOU=============="
#         print "OU=",self.GetSubjectOU()
#         print "===========GetIssuerX500Principal============"
#         print "Issuer:",self.GetIssuerX500Principal()
        
    def AnalysisRDNSequence(self, RDNSequence):
        psList = {}
        OUCount = 0
        for RelativeDistinguishedName in RDNSequence:
            RDN = RelativeDistinguishedName[0]  #类实例，pyasn1_modules.rfc2459.AttributeTypeAndValue
            AttributeType = RDN['type']         #类实例，pyasn1_modules.rfc2459.AttributeType
            AttributeValue = RDN['value']       #类实例，pyasn1_modules.rfc2459.AttributeValue
            valList = decoder.decode(AttributeValue)
            ps = str(valList[0])    # pyasn1.type.char.PrintableString
            av = valList[1]         # pyasn1_modules.rfc2459.AttributeValue
            #print "ps=", ps
            #print "av=", av
            if(AttributeType == rfc2459.id_at_organizationalUnitName):
                OUCount += 1
                if(OUCount == 1):
                    psList['OU1'] = ps
                    continue
                elif(OUCount == 2):
                    psList['OU2'] = ps
                    continue
            psList[id[AttributeType]] = ps
        return psList        
        
    def GetVersion(self):
        version = self.cert['tbsCertificate']['version']
        return version
    
    def GetSerialNumber(self):
        serialNumberInt = self.cert['tbsCertificate']['serialNumber']
        serialNumberBytes = number.long_to_bytes(serialNumberInt)
        serialNumberHex = Codeutil.toHexString(serialNumberBytes)
        return serialNumberHex

    def GetIssuerX500Principal(self):
        IssuerDic = {}
        RDNSequence = self.cert['tbsCertificate']['issuer'][0]
        IssuerDic = self.AnalysisRDNSequence(RDNSequence)
        return IssuerDic
        
    def GetTbsCertificate(self):
        return encoder.encode(self.cert['tbsCertificate'])
    
    def GetCertSign(self):
        SigAlgDic = {}
        signatureAlgorithm = self.cert['signatureAlgorithm']
        algorithm = signatureAlgorithm['algorithm']
        signatureValue = self.cert['signatureValue']
        SigAlgDic['oid'] = str(algorithm)   #获取证书的签名算法 OID 字符串
        SigAlgDic['name'] = id[algorithm]   #获取证书签名算法的签名算法名
        SigAlgDic['value'] = Codeutil.toHexString(valueOnlyBitStringEncoder(signatureValue))
        return SigAlgDic
        
    def GetSignature(self):
        return valueOnlyBitStringEncoder(self.cert['signatureValue'])
    
    def GetSignatureValue(self):
        return encoder.encode(self.cert['signatureValue'])
        
    def GetSubjectPublicKeyInfo(self):
        SPKIDic = {}
        spki = self.cert['tbsCertificate']['subjectPublicKeyInfo']
        algorithmTupe = spki['algorithm']
        pubkey = spki['subjectPublicKey']
        
        algType = algorithmTupe[0]
        SPKIDic['algorithm'] = id[algType]  #加密算法
        
        pk = valueOnlyBitStringEncoder(pubkey)
        pkSeqenceTuple = decoder.decode(pk)
        modulus = number.long_to_bytes(pkSeqenceTuple[0][0])        #n
        publicExponent = number.long_to_bytes(pkSeqenceTuple[0][1]) #e
        SPKIDic['n'] = Codeutil.toHexString(modulus)
        SPKIDic['e'] = Codeutil.toHexString(publicExponent)
        return SPKIDic
        
    def GetPublicKey(self):
        spki = self.cert['tbsCertificate']['subjectPublicKeyInfo']
        return valueOnlyBitStringEncoder(spki['subjectPublicKey'])
    
    def GetSubjectDN(self):
        RDNSequence = self.cert['tbsCertificate']['subject'][0]
        DNDic = {}
        DNDic = self.AnalysisRDNSequence(RDNSequence)
        return DNDic 
    
    def GetSubjectOU(self):
        DNDic = self.GetSubjectDN()
        return DNDic['OU1']
        
    def VerifyCert(self, CACert):
        ohash = SHA.new(self.GetTbsCertificate()).digest()
        cakey = RSA.importKey(CACert.GetSubjectPublicKeyInfo())
        shash = cakey.encrypt(self.GetSignature(),'')[0]
        if ohash == shash[-20:]:
            return True
            
        return False
    
def toDerFmt(input):
    '''@input --certficate file data to be transfer
        base64-->der
    '''
    
    file_start_flag = "-----BEGIN CERTIFICATE-----"
    file_end_flag = "-----END CERTIFICATE-----"
    
    #base64 format
    s_index = input.find(file_start_flag)
    if s_index == 0:
        e_index = input.find(file_end_flag)
        if(e_index != -1):
            data = input[s_index + len(file_start_flag) : e_index]
            output = base64.decodestring(data)
        else:
            return None
    else:
    #not base64 format
        output = input
    return output
    



#! python
#
# Python Serial Port Extension for Win32, Linux, BSD, Jython
# see ../__init__.py
#
# This is a thin wrapper to load the rfc2271 implementation.
#
# (C) 2011 Chris Liechti <cliechti@gmx.net>
# this is distributed under a free software license, see license.txt

from serial27.rfc2217 import Serial



#coding:utf-8

import time
from threading import Thread, Event
from utilityhelper.util.Observer import *
from utilityhelper.common import HelperDecorator
from utilityhelper.helperserial.serial_port_manager import *

_START_ON_DEMAND_ = False


@HelperDecorator.singleton
class SerialReaderMonitor(Observable):
    """"""
    def __init__(self):
        Observable.__init__(self)
        if _START_ON_DEMAND_:
            self.rmthread = None
        else:
            self.rmthread = _SerialReaderMonitoringThread(self)
            self.rmthread.start()

    def addObserver(self, observer):
        Observable.addObserver(self, observer)
        if _START_ON_DEMAND_:
            if self.countObservers() > 0 and self.rmthread is None:
                self.rmthread = _SerialReaderMonitoringThread(self)
                self.rmthread.start()
        else:
            if self.rmthread.devs:
                observer.update(self, (self.rmthread.devs, []))

    def deleteObserver(self, observer):
        """Remove an observer.

        We delete the _BLESerialDevMonitoringThread reference when there
        are no more observers.
        """
        Observable.deleteObserver(self, observer)
        if _START_ON_DEMAND_:
            if self.countObservers() == 0:
                if self.rmthread is not None:
                    self.rmthread = None

@HelperDecorator.singleton
class _SerialReaderMonitoringThread(Thread):
    """Card insertion thread.
    This thread waits for card insertion.
    """
    def __init__(self, observable):
        Thread.__init__(self)
        self.observable = observable
        self.stopEvent = Event()
        self.stopEvent.clear()
        self.devs = []
        self.setDaemon(True)

    # the actual monitoring thread
    def run(self):
        while self.stopEvent.isSet() != 1:
            time.sleep(0.5)
            try:
                currentdevs = SerialPortManager.find_port()

                addeddevs = []
                for dev in currentdevs:
                    if not self.devs.__contains__(dev):
                        addeddevs.append(dev)

                removeddevs = []
                for dev in self.devs:
                    if not currentdevs.__contains__(dev):
                        removeddevs.append(dev)

                if addeddevs != [] or removeddevs != []:
                    self.devs = currentdevs
                    self.observable.setChanged()
                    self.observable.notifyObservers((addeddevs, removeddevs))
            except TypeError:
                    pass
            except AttributeError:
                    pass
            except Exception as e:
                pass

    # stop the thread by signaling stopEvent
    def stop(self):
        self.stopEvent.set()


class SerialReaderObserver(Observer):
    """串口读卡器设备检测"""

    def __init__(self, observable, callback):
        """实例化观察者
        :param observable:被观察对象
        :param callback: 观察者注册的事件回调方法，当方法中有UI操作时的，必须使用异步调用，用相应的跨线程处理机制包装。
                          1）wxpython，使用wx.CallAfter方法包装；
                          2）pyqt，使用QMetaObject.invokeMethod方法包装
        """
        self.callback = callback
        self.observable = observable
        # 新建观察者默认添加到被观察者维护的队列中
        observable.addObserver(self)

    def update(self, observable, args):
        plugins, plugouts = args
        self.callback(plugins, plugouts)




#coding:utf-8
from __future__ import (print_function, unicode_literals)
from utilityhelper.compatible import *
import logging
import logging.handlers
import os

class EncodingFormatter(logging.Formatter):

    def __init__(self, fmt, datefmt=None, encoding=None):
        super(EncodingFormatter, self).__init__(fmt, datefmt)
        self.encoding = encoding

    def format(self, record):
        result = logging.Formatter.format(self, record)
        if not PY3:
            if isinstance(result, unicode):
                result = result.encode(self.encoding or 'utf-8')
        return result

class Log(object):
    
    currScriptPath = os.path.split(os.path.realpath(__file__))[0]
    lifile = os.listdir(currScriptPath)
    os.chdir(currScriptPath)
    logfile = r"testLog.log"
    logger=logging.getLogger("testLog")
    logger.setLevel(logging.DEBUG)
    # 创建一个handler，用于写入日志文件
    # handler=logging.FileHandler(logfile)
    handler=logging.StreamHandler()

    # 定义handler的输出格式
    fmtstr = '[%(asctime)s] [%(levelname)s] [%(name)s] [PID:%(process)d-TID:%(thread)d(%(threadName)s)] -- %(message)s'
    formatter = EncodingFormatter(fmtstr)
    handler.setFormatter(formatter)
    logger.addHandler(handler)
    
    @staticmethod
    def SetLevel(level):
        Log.logger.setLevel(level)
    
    @staticmethod
    def GetLogLevel():
        return Log.logger.level
        
    @staticmethod
    def Info(msg):
        Log.logger.info(msg)
        
    @staticmethod        
    def Debug(msg):
        Log.logger.debug(msg)
         
    @staticmethod
    def Warn(msg):
        Log.logger.warning(msg)
            
    @staticmethod       
    def Error(msg):
        Log.logger.error(msg)
    

class MailLog(Log):
    logger=logging.getLogger("mailFactory")
    logger.setLevel(logging.DEBUG)
    #创建一个SMTP handler，用于输出日志到邮件
    mailHandler = logging.handlers.SMTPHandler(mailhost="",
                                     fromaddr='',
                                     toaddrs='',
                                     subject='Logged Event',
                                     credentials = ('', ''))

    mailHandler.setFormatter(EncodingFormatter(Log.fmtstr, encoding='utf-8'))
    logger.addHandler(mailHandler)

    @staticmethod
    def SetLevel(level):
        MailLog.logger.setLevel(level)
    
    @staticmethod
    def GetLogLevel():
        return MailLog.logger.level
        
    @staticmethod
    def Info(msg):
        MailLog.logger.info(msg)
        
    @staticmethod        
    def Debug(msg):
        MailLog.logger.debug(msg)
         
    @staticmethod
    def Warn(msg):
        MailLog.logger.warning(msg)
            
    @staticmethod       
    def Error(msg):
        MailLog.logger.error(msg)

if __name__ == "__main__":
    Log.Info("11111111111222222222222222")
    Log.Info("UNICODE中文")


#coding:utf-8
#一个关于二维列表的通用的表
'''
GenericTable类要求一个数据的二维列表和一个可选的行和列标签列表。这
个类适合被导入任何wxPython程序中。使用一个做了微小改变的格式.
'''
import wx
import wx.grid

class GenericTable(wx.grid.GridTableBase):
    def __init__(self, data, colDataTypes= [], rowLabels=None, colLabels=None, onGridValueChanged=None):
        wx.grid.GridTableBase.__init__(self)
        self.data = data
        self.dataTypes = []
        if(colDataTypes == []):
            for i in range(len(self.data[0])):
                self.dataTypes.append(wx.grid.GRID_VALUE_STRING) 
        else:
            self.dataTypes = colDataTypes
            for i in range(len(self.data[0]) - len(colDataTypes)):
                self.dataTypes.append(wx.grid.GRID_VALUE_STRING)
        self.rowLabels = rowLabels
        self.colLabels = colLabels
        self.isModified = False
        self.onGridValueChanged = onGridValueChanged
    
    #---------------------------------------------------------------------
    '''    
    five required methods for the wxPyGridTableBase interface 
    '''  
    def GetNumberRows(self):
        if self.data is None:
            return 0
        return len(self.data)
    

    def GetNumberCols(self):
        """"""
        if self.colLabels is None or len(self.colLabels) == 0:
            return 0
        return len(self.data[0])
    
    def IsEmptyCell(self, row, col):
        try:
            return not self.data[row][col]
        except IndexError:
            return True
    
    '''Get/Set values in the table.  The Python version of these
    methods can handle any data-type, (as long as the Editor and
    Renderer understands the type too,) not just strings as in the
    C++ version.'''
    def GetValue(self, row, col):
        try:
            return self.data[row][col]
        except IndexError:
            return ''
            
    def SetValue(self, row, col, value):
        def innerSetValue(row, col, value):
            try:
                self.data[row][col] = value
                self.isModified = True
                if self.onGridValueChanged:
                    self.onGridValueChanged()
            except IndexError:
                # add a new row
                self.data.append([''] * self.GetNumberCols())
                innerSetValue(row, col, value)
                # tell the grid we've added a row
                msg = wx.grid.GridTableMessage(self,            # The table
                        wx.grid.GRIDTABLE_NOTIFY_ROWS_APPENDED, # what we did to it
                        1                                       # how many
                        )
                self.GetView().ProcessTableMessage(msg)
        innerSetValue(row, col, value) 
        
#----------------------------------------------------------------------
# Some optional methods
    def GetColLabelValue(self, col):
        '''Called when the grid needs to display labels'''
        if self.colLabels:
            return self.colLabels[col]
    
    def GetRowLabelValue(self, row):
        if self.rowLabels:
            return self.rowLabels[row]
            
    # Called to determine the kind of editor/renderer to use by
    # default, doesn't necessarily have to be the same type used
    # natively by the editor/renderer if they know how to convert.
    def GetTypeName(self, row, col):
        if self.dataTypes:
            return self.dataTypes[col]
    
    # Called to determine how the data can be fetched and stored by the
    # editor and renderer.  This allows you to enforce some type-safety
    # in the grid.
    def CanGetValueAs(self, row, col, typeName):
        colType = self.dataTypes[col].split(':')[0]
        if typeName == colType:
            return True
        else:
            return False
        
    def CanSetValueAs(self, row, col, typeName):
        return self.CanGetValueAs(row, col, typeName)
    
    def InsertRows(self,pos=1,numRows=1):
        """"""
        for num in range(0,numRows):
            newData={};
            newData['lable'] = ''     
            self.data.insert(pos,newData)
            
        self.isModified = True
        gridView = self.GetView()
        gridView.BeginBatch()
        insertMsg = wx.grid.GridTableMessage(self,wx.grid.GRIDTABLE_NOTIFY_ROWS_INSERTED,pos,numRows)
        gridView.ProcessTableMessage(insertMsg)
        gridView.EndBatch()
        getValueMsg = wx.grid.GridTableMessage(self,wx.grid.GRIDTABLE_REQUEST_VIEW_GET_VALUES)
        gridView.ProcessTableMessage(getValueMsg)        
        
        if self.onGridValueChanged:
            self.onGridValueChanged()        
        return True
   
    def AppendRows(self,numRows=1):
        """"""
        for num in range(0,numRows):
            newData={};
            newData['lable'] = ''         
            self.data.append(newData)
        self.isModified = True
        gridView = self.GetView()
        gridView.BeginBatch()
        appendMsg = wx.grid.GridTableMessage(self,wx.grid.GRIDTABLE_NOTIFY_ROWS_APPENDED,numRows)
        gridView.ProcessTableMessage(appendMsg)
        gridView.EndBatch()
        getValueMsg = wx.grid.GridTableMessage(self,wx.grid.GRIDTABLE_REQUEST_VIEW_GET_VALUES)
        gridView.ProcessTableMessage(getValueMsg)
        
        if self.onGridValueChanged:
            self.onGridValueChanged()  
            
        return True
    
    def DeleteRows(self,pos=0,numRows=1):
        if self.data is None or len(self.data) == 0:
            return False
        for rowNum in range(0, numRows):
            self.data.remove(self.data[pos+rowNum -1])

        gridView = self.GetView()
        gridView.BeginBatch()
        deleteMsg = wx.grid.GridTableMessage(self,wx.grid.GRIDTABLE_NOTIFY_ROWS_DELETED,pos,numRows)
        gridView.ProcessTableMessage(deleteMsg)
        gridView.EndBatch()
        getValueMsg = wx.grid.GridTableMessage(self,wx.grid.GRIDTABLE_REQUEST_VIEW_GET_VALUES)
        gridView.ProcessTableMessage(getValueMsg)
  
        if self.onGridValueChanged:
            self.onGridValueChanged()
            
        return True    




#coding:utf-8
from __future__ import (print_function, unicode_literals)
import os
import time
try:
    import MySQLdb
except:
    import pymysql
    pymysql.install_as_MySQLdb()

class Singleton(type):    
    def __init__(cls, name, bases, dict):    
        super(Singleton, cls).__init__(name, bases, dict)    
        cls._instance = None
            
    def __call__(cls, *args, **kw):    
        if cls._instance is None:    
            cls._instance = super(Singleton, cls).__call__(*args, **kw)    
        return cls._instance


class MySQLUtil(object, metaclass=Singleton):
    '''
                对MySQLdb常用函数进行封装的类
    '''    
    _instance = None #本类的实例
    _conn = None # 数据库conn
    _cur = None #游标
    error_code = '' #MySQL错误号码
       
    def __init__(self, dbconfig):
        
        '''构造器：根据数据库连接参数，创建MySQL连接'''
        try:    
#             self._timecount = 0
#             self._TIMEOUT = 10
            self._conn = MySQLdb.connect(host=dbconfig['host'],
                                           port=dbconfig['port'], 
                                           user=dbconfig['user'],
                                           passwd=dbconfig['passwd'],
                                           db=dbconfig['db'],
                                           charset=dbconfig['charset'])
            self._cur = self._conn.cursor()
            
            print("self._conn=", self._conn)
            print("self._cur=", self._cur)
            print("self._instance=", self._instance)
            
        except MySQLdb.Error as e:
            self.error_code = e.args[0]
            error_msg = str(e.args[0]) + " - "+ e.args[1]
            print("e=", e)
            print("MySQLdb.Error=",MySQLdb.Error)
            print("error_msg=", error_msg)
            raise DBUtilError(e) 
            #raise Exception(error_msg)
          
            # 如果没有超过预设超时时间，则再次尝试连接，
#             if self._timecount < self._TIMEOUT:
#                 interval = 5
#                 self._timecount += interval
#                 time.sleep(interval)
#                 return self.__init__(dbconfig)
#             else:
#                 raise Exception(error_msg)

    def query(self,sql):
        '''执行 SELECT 语句'''     
        try:
            self._cur.execute("SET NAMES utf8") 
            result = self._cur.execute(sql)
        except MySQLdb.Error as e:
            self.error_code = e.args[0]
            print("query error:",e.args[0],e.args[1])
            result = False
            raise DBUtilError(e)
        return result

    def update(self,sql):
        '''执行 UPDATE 及 DELETE 语句'''
        try:
            self._cur.execute("SET NAMES utf8") 
            result = self._cur.execute(sql)
            self._conn.commit()
        except MySQLdb.Error as e:
            self.error_code = e.args[0]
            print("数据库更新错误,错误代码:",e.args[0],e.args[1])
            result = False
            raise DBUtilError(e)
        return result
        
    def insert(self,sql):
        '执行 INSERT 语句。如主键为自增长int，则返回新生成的ID'
        try:
            self._cur.execute("SET NAMES utf8")
            self._cur.execute(sql)
            self._conn.commit()
            return self._conn.insert_id()
        except MySQLdb.Error as e:
            self.error_code = e.args[0]
            print("数据库insert错误,错误代码:",e.args[0],e.args[1])
            raise DBUtilError(e)
            #return False
    
    def insertMany(self, sql, dataTuple):
        '''                
                            执行 批量INSERT 语句。如主键为自增长int，则返回新生成的ID
        '''
        try:
            self._cur.execute("SET NAMES utf8")
            self._cur.executemany(sql, dataTuple)
            self._conn.commit()
            return self._conn.insert_id()
        except MySQLdb.Error as e:
            self.error_code = e.args[0]
            print("数据库insertMany错误,错误代码:",e.args[0],e.args[1])
            return False 

    def deleteMany(self, sql, dataTuple):
        '执行 批量INSERT 语句。如主键为自增长int，则返回新生成的ID'
        try:
            self._cur.execute("SET NAMES utf8")
            self._cur.executemany(sql, dataTuple)
            self._conn.commit()
        except MySQLdb.Error as e:
            self.error_code = e.args[0]
            print("数据库deleteMany错误,错误代码:",e.args[0],e.args[1])
            return False 
                     
    def fetchAllRows(self):
        '返回结果列表'
        return self._cur.fetchall()
    
    def fetchOneRow(self):
        '返回一行结果，然后游标指向下一行。到达最后一行以后，返回None'
        return self._cur.fetchone()
     
    def getRowCount(self):
        '获取结果行数'
        return self._cur.rowcount
    
    def getTableHead(self):
        #cursor结果集合的描述（列名字，数据类型，是否允许空，列标志）
        #type: 3(LONG), 253(VAR_STRING)，10（DATE）
        #是否允许空：0
        #列标志：20483， 4097， 4225
        head_list = []
        desc = self._cur.description
        if(desc != None):
            #推倒式列表
            head_list = [col[0] for col in desc]
        return head_list
                  
    def commit(self):
        '数据库commit操作'
        self._conn.commit()
                
    def rollback(self):
        '数据库回滚操作'
        self._conn.rollback()
           
    def __del__(self): 
        '释放资源（系统GC自动调用）'
        try:
            print("__del__ ", self)
            self._cur.close() 
            self._conn.close() 
        except:
            pass
        
    def getConnection(self):
        return self._conn
        
    def close(self):
        '关闭数据库连接'
        self.__del__()
    
    @classmethod
    def getInstance(cls):
        return cls._instance


class DBUtilError(Exception):
    def __init__(self, e):
        Exception.__init__(self)
        self.errID = e.args[0]
        self.errMsg = e.args[1]
        self.errInfo = str(self.errID) + " - " + self.errMsg
    
    def __str__(self, *args, **kwargs):
        return self.errInfo



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import codecs
import os
import sys
import platform
from shutil import rmtree

# from distutils.core import setup, Command
from setuptools import setup, find_packages, Command

from distutils.command.build        import build as orig_build
from distutils.command.install      import install as orig_install
# from setuptools.command.install     import install as orig_install
from setuptools.command.bdist_egg   import bdist_egg as orig_bdist_egg
from setuptools.command.sdist       import sdist as orig_sdist
try:
    from wheel.bdist_wheel import bdist_wheel as orig_bdist_wheel
    haveWheel = True
except ImportError:
    haveWheel = False

here = os.path.abspath(os.path.dirname(__file__))

with codecs.open(os.path.join(here, "README.md"), encoding="utf-8") as f:
    long_description = "\n" + f.read()

about = {}

with open(os.path.join(here, "utilityhelper", "__version__.py")) as f:
    exec(f.read(), about)

setup_required = [
    "pbr"
]

install_required = [
    "pip>=9.0.1",
    "setuptools>=36.2.1",
]

if platform.platform().startswith("Windows"):
    os_maj_ver = int(platform.win32_ver()[1].split('.')[0])
    if int(os_maj_ver) >= 6:
        install_required.append("pyserial>2.7")  # windows vista and later
    else:
        install_required.append("pyserial<=2.7")

# https://pypi.python.org/pypi/stdeb/0.8.5#quickstart-2-just-tell-me-the-fastest-way-to-make-a-deb
class DebCommand(Command):
    """Support for setup.py deb"""

    description = "Build and publish the .deb package."
    user_options = []

    @staticmethod
    def status(s):
        """Prints things in bold."""
        print("\033[1m{0}\033[0m".format(s))

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            self.status("Removing previous builds…")
            rmtree(os.path.join(here, "deb_dist"))
        except Exception as e:
            pass
        self.status("Creating debian mainfest…")
        os.system(
            "python setup.py --command-packages=stdeb.command sdist_dsc -z artful --package3=pipenv --depends3=python3-virtualenv-clone"
        )
        self.status("Building .deb…")
        os.chdir("deb_dist/pipenv-{0}".format(about["__version__"]))
        os.system("dpkg-buildpackage -rfakeroot -uc -us")


if haveWheel:
    class wx_bdist_wheel(orig_bdist_wheel):
        def finalize_options(self):
            # Do a bit of monkey-patching to let bdist_wheel know that there
            # really are extension modules in this build, even though they are
            # not built here.
            def _has_ext_modules(self):
                return True
            from setuptools.dist import Distribution
            #Distribution.is_pure = _is_pure
            Distribution.has_ext_modules = _has_ext_modules

            orig_bdist_wheel.finalize_options(self)


        def run(self):
            # Ensure that there is a basic library build for bdist_egg to pull from.
            self.run_command("build")

            # Run the default bdist_wheel command
            orig_bdist_wheel.run(self)


class UploadCommand(Command):
    """Support setup.py upload."""

    description = "Build and publish the package."
    user_options = []

    @staticmethod
    def status(s):
        """Prints things in bold."""
        try:
            print("\033[1m{0}\033[0m".format(s))
        except:
            pass

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            self.status("Removing previous builds…")
            rmtree(os.path.join(here, "dist"))
        except Exception as e:
            pass
        self.status("Building Source distribution…")
        os.system("{0} setup.py sdist bdist_wheel".format(sys.executable))
        self.status("Uploading the package to PyPI via Twine…")
        os.system("twine upload dist/*")
        self.status("Pushing git tags…")
        os.system("git tag v{0}".format(about["__version__"]))
        os.system("git push --tags")
        sys.exit()


# Map these new classes to the appropriate distutils command names.

CMDCLASS = {
    'build'       : orig_build,
    'bdist_egg'   : orig_bdist_egg,
    'install'     : orig_install,
    'sdist'       : orig_sdist,
    'upload'      : UploadCommand,
    'deb'         : DebCommand,
    }

# if haveWheel:
#     CMDCLASS['bdist_wheel'] = wx_bdist_wheel

# WARNING: Uploading via this command is deprecated, use twine to upload instead (https://pypi.org/p/twine/)
# error: Must create and upload files in one command (e.g. setup.py sdist upload)
# if sys.argv[-1] == "publish":
#     os.system("python setup.py sdist bdist_wheel upload")
#     sys.exit()

setup(
    name="utilityhelper",
    version=about["__version__"],
    description="assistant tool for coding",
    long_description=long_description,
    long_description_content_type='text/markdown',
    author="Lemon Gong",
    author_email="gll72344@gmail.com",
    url="https://github.com/leileigong/utility-helper",
    packages=find_packages(exclude=[]),
    entry_points={
    },
    package_data={
        "": ["LICENSE", "NOTICES"],
    },
    python_requires=">=2.7,!=3.0.*,!=3.1.*,!=3.2.*,!=3.3.*",
    setup_requires=setup_required,
    install_requires=install_required,
    extras_require={},
    include_package_data=True,
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: Implementation :: CPython",
    ],
    # pbr=True,
    platforms='any',
    cmdclass=CMDCLASS,
)






#coding:utf-8
"""
binascii模块:二进制和ASCII互转.Python版本：1.5及以后版本
    1)包含很多在二进制和ASCII编码的二进制表示转换的方法;
    2)包含更高级别的模块使用的，用C语言编写的低级高效功能;
    3)偶有使用于字符串和ASCII的转换;
通常情况不会直接使用这些功能，而是使用像UU，base64编码，或BinHex封装模块。
"""
import six
import binascii
import struct
import base64


def string_to_string_list(strs, sep=' '):
    """字符串转列表。根据指定字符串sep拆分字符串
    >>> s = 'a b c'
    >>> string_to_string_list(s)
    ['a', 'b', 'c']
    >>> string_to_string_list('x11x22x33', 'x')
    ['', '11', '22', '33']
    """
    return [s for s in strs.split(sep)]


def string_list_to_string(stringli, sep=''):
    """列表转字符串
    :param stringli:
    :param sep:
    :return:
    >>> string_list_to_string(['1', '2', '3', '4', '5'])
    '12345'
    >>> string_list_to_string(['1', '2', '3', '4', '5'], '.')
    '1.2.3.4.5'
    >>> string_list_to_string(['1', '2', '3', '4', '5'], ' ')
    '1 2 3 4 5'
    """
    return sep.join(stringli)


def int_list_to_string(intli, sep=b''):
    """整数列表转字符串
    :param intli:
    :param sep:
    :return:
    >>> int_list_to_string([1,2])
    '\\x01\\x02'
    """
    return sep.join([six.int2byte(n) for n in intli])

def structedhex_string_to_bytes(hexString, sep1, sep2):
    """十六进制字符串格式化成带特定分隔符的二进制
    :param hexString:
    :param sep1:
    :param sep2:
    :return:
    >>> structedhex_string_to_bytes("AA:BB:11:22:33", ':', '')
    '\\xaa\\xbb\\x11"3'
    >>> structedhex_string_to_bytes("31:32:33:34:35", ':', '*')
    '1*2*3*4*5'
    >>> structedhex_string_to_bytes("31:32:33:34:35", ':', '*')
    '1*2*3*4*5'
    """
    return sep2.join([chr(int(c, 16)) for c in hexString.split(sep1)])


def bytes_to_string_list(bytes):
    """
    :param bytes:
    :return:
    >>> bytes_to_string_list('\x12\x34')
    ['\\x12', '4']
    """
    li = []
    li = map(lambda c: c, bytes)
    return li


def bytes_to_int_list(bytes):
    """bytesToIntergerList
    :param bytes:
    :return:
    >>> bytes_to_int_list("12")
    [49, 50]
    """
    li = map(lambda c: ord(c), bytes)
    return li

# 过时的方法名
bytesToIntergerList = bytes_to_int_list

def bytes_to_wint_list(bytes, bitwidth=8, edian='B'):
    """bytesToIntList
    Args:
        bytes:
        bitwidth:
        edian:

    Returns:
    >>> bytes_to_wint_list("12")
    [49, 50]
    >>> s = '\x01\x02\x03\x04\x05\x06\x07\x08'
    >>> bytes_to_wint_list(s, bitwidth=8)
    [1, 2, 3, 4, 5, 6, 7, 8]
    >>> bytes_to_wint_list(s, bitwidth=16)
    [258, 772, 1286, 1800]
    >>> bytes_to_wint_list(s, bitwidth=32)
    [16909060, 84281096]
    """
    li = []
    if bitwidth==8:
            tp = struct.unpack("B" * len(bytes), bytes)
            li = list(tp)
    elif bitwidth==16: # short
        if edian.upper() == 'B':
            tp = struct.unpack(("!" + "H" * (len(bytes)/2)), bytes)
            li = list(tp)
        else:
            tp = struct.unpack(("H" * (len(bytes) / 2)), bytes)
            li = list(tp)
    elif bitwidth==32: # int or long
        if edian.upper() == 'B':
            tp = struct.unpack(("!" + "I" * (len(bytes)/4)), bytes)
            li = list(tp)
        else:
            tp = struct.unpack(("I" * (len(bytes)/4)), bytes)
            li = list(tp)

    return li


def int_list_to_bytes(bytes):
    """
    ASCII码值列表转字符串
    :param bytes:
    :return:
    >>> int_list_to_bytes([48,49])
    '01'
    >>> int_list_to_bytes([0x48,0x49])
    'HI'
    """
    return ''.join(map(chr, bytes))


bytesToString = int_list_to_bytes

def string_split_by_length(data, step):
    """按照给定的宽度切割字符串
    :param data:
    :param step:
    :return:
    >>> string_split_by_length("123", 1)
    ['1', '2', '3']
    >>> string_split_by_length("123", 2)
    ['12', '3']
    >>> string_split_by_length(b"123", 3)
    ['123']
    """
    hexlist = [data[i:i + step] for i in xrange(0, len(data), step)]
    return hexlist


def bytes_to_hex_string(strs, sep=''):
    """bytes_to_hex_string
    二进制字符串格式化成十六进制字符串；字符串转ASCII码；
    并使用连接字符串sep连接
    :param strs:
    :param sep:
    :return:
    >>> bytes_to_hex_string('hello')
    '68656C6C6F'
    >>> bytes_to_hex_string('\x11\x22')
    '1122'
    >>> bytes_to_hex_string(",\xFF,\xFF", '')
    '2CFF2CFF'
    """
    if sep != '':
        hexstring = sep.join(map(lambda c: "%02X" % ord(c), strs))
    else:
        # hexstring = binascii.hexlify(data)
        hexstring = binascii.b2a_hex(strs).upper()
    return hexstring

# 过时的方法名
BytesToFormatHexString = bytes_to_hex_string

def hex_string_to_bytes(strHex, sep=''):
    """hex_string_to_bytes
    十六进表示的字符串转二进制；ASCII码转字符串；
    使用连接字符串sep连接
    :param strHex:
    :param sep:
    :return:
    >>> hex_string_to_bytes('68656c6c6f')
    'hello'
    >>> hex_string_to_bytes('313233')
    '123'

    """
    if sep != '':
        binstr = sep.join(map(lambda h: chr(int(h, 16)), string_split_by_length(strHex, 2)))
    else:
        # binstr =  binascii.unhexlify(strHex)
        binstr = binascii.a2b_hex(strHex)
    return binstr

hexToString = hex_string_to_bytes


def hex_string_to_int_list(strHex):
    """十六进制字符串转整数列表
    :param strHex:
    :return:
    >>> hex_string_to_int_list('12ab34')
    [18, 171, 52]
    """
    return map(lambda h: int(h, 16), string_split_by_length(strHex, 2))


hexToIntList = hex_string_to_int_list

def hex_string_to_hex_list(strHex):
    """十六进制字符串转为列表
    :param strHex:
    :return:
     >>> hex_string_to_hex_list('12ab34')
     ['0x12', '0xab', '0x34']
    """
    return map(lambda h: hex(int(h, 16)), string_split_by_length(strHex, 2))


def int_number_to_hex_string(num, strlen):
    """把整数格式化成指定长度的十六进制字符串
    :param num:
    :param strlen:
    :return:
    >>> int_number_to_hex_string(65535, 4)
    'FFFF'
    >>> int_number_to_hex_string(65536, 8)
    '00010000'
    """
    return ''.join(("%" + "0%d" % strlen + "X") % num)


def bcd_string_to_int_number(bcdBytes):
    """8421BCD码与十进制数字字符串之间的转换。
    :param bcdBytes:
    :return:
    >>> bcd_string_to_int_number('\x12\x34')
    1234
    >>> bcd_string_to_int_number('1234')
    31323334
    >>> bcd_string_to_int_number('0023')
    30303233
    """
    IntVal = 0
    for (i, val) in enumerate(bcdBytes):
        # 这种编码形式利用四个位来储存一个十进制数,4位一组取值
        IntVal = IntVal * 10 + (ord(val) >> 4)      # 取字节高4bits
        IntVal = IntVal * 10 + (ord(val) & 0x0F)    # 取字节低4bits
    return IntVal


def int_number_to_bcd_string(integer):
    """整数转8421BCD码
    :param int:
    :return:
    >>> int_number_to_bcd_string(100)
    '000100000000'
    >>> int_number_to_bcd_string(65535)
    '01100101010100110101'
    """
    bcd = ''
    s = str(integer)
    for _ in s:
        bcd += '{0:0>4}'.format(bin(int(_)).replace('0b', ''))
    return bcd


def string_xor(str):
    """16进制字符串取反
    >>> string_xor('1234')
    'EDCB'
    >>> string_xor('FFFF')
    '0000'
    """
    by = 0
    bytes = hex_string_to_bytes(str)
    rbytes = ''
    for i in range(0,len(bytes)):
        rbytes += unichr((~ord(bytes[i]))&0xFF)
    return bytes_to_hex_string(rbytes)


strXor = string_xor


#for python 2
def _is_stringish(x):
    return isinstance(x, (str, basestring, bytes, unicode))


def _is_unicode(val):
    return isinstance(val, unicode)


def _is_intish(x):
    return isinstance(x, (int, long))


def tuple_to_string(tup_):
    """tuple2str
    :param tup_:
    :return:
    >>> tuple_to_string((1,2))
    '(1, 2, )'
    """
    s = ''
    s += '('
    for i in tup_:
        if _is_stringish(i):
            if _is_stringish(i):
                s += '\''
                if _is_unicode(i):
                    s += i.encode('utf8')
                else:
                    s += i
                s += '\''
        elif _is_intish(i):
            s += str(i)
        elif isinstance(i, list):
            s += list_to_string(i)
        s += ', '
    s += ')'
    return s


def list_to_string(li_):
    """list2str
    :param li_:
    :return:
    """
    s = '['
    for i in li_:
        if _is_stringish(i):
            s += '\''
            if _is_unicode(i):
                s += i.encode('utf8')
            else:
                s += i
            s += '\''
        elif _is_intish(i):
            s + str(i)
        elif isinstance(i, list):
            s += list_to_string(i)
        s += ', '
    s += ']'
    return s


def dict_to_string(dict_):
    """dict2str
    :param dict_:
    :return:
    """
    s = '{'
    for k, v in dict_.iteritems():
        # key
        # key type is str or unicode or int, tuple, fun
        if _is_stringish(k):
            s += '\''
            if _is_unicode(k):
                s += k.encode('utf8')
            else:
                s += k
            s += '\''
        elif _is_intish(k):
            s += str(k)
        elif isinstance(k, tuple):
            s += tuple_to_string(k)
        s += ": "

        # value
        if _is_stringish(v):
            s += '\''
            if _is_unicode(v):
                s += v.encode('utf8')
            else:
                s += v
            s += '\''
        elif _is_intish(v):
            s += str(v)
        elif isinstance(v, list):
            s += list_to_string(v)
        s += ', '
    s += '}'
    return s


def struct_data_to_string(data):
    """
    结构化的数据转为字符串，主要用于当数据对象里的元素包含中文的情况下，使用print打印不能输出中文的情况。
    支持多层嵌套，如{[{}, {}], {}, binary string, unicode string}
    :param data:
    :return:
    """
    s = ''
    if isinstance(data, dict):
        s += dict_to_string(data)
    elif isinstance(data, list):
        s += list_to_string(data)
    elif isinstance(data, tuple):
        s += tuple_to_string(data)
    return s


def hex_string_base64_encode(HexString):
    """Base64Encoder
    :param HexString:
    :return:
    """
    s = hex_string_to_bytes(HexString)
    return base64.b64encode(s)


def hex_string_base64_decode(HexString):
    s = hex_string_to_bytes(HexString)
    return base64.b64decode(s)


if __name__ == '__main__':
    dic = {
            'test str key1': u'测试Unicode值1', '测试str键1': u'test str value1', '测试str键2': ['', '测试str值2'],
            u'测试Unicode键1': ['', '测试str值2'],
            u'测试Unicode键2': ['', u'测试Unicode值2'],
            u'测试混合值': ['', {u'嵌套建1' : u'嵌套值1'}],
           (u'列表混合键1-Unicode', '列表混合键2-str'): u'测试混合键',
           }

    # print struct2str(dic)
    # print Base64Encoder("000345d2da3fce")
    # print Base64Encoder("0003376CD12FDE")
    # s = "414542474D502F2F2F2F2F2F2F2F2F2F2F2F395552554E495530684A546B386751554A4461476C755953417449455A70626D646C6369424C5A586C5164574A7361574D67566D567963326C76626942574D6A41314145726D7748303D"
    # print hexToString(s)
    print(bytes_to_hex_string(hex_string_base64_decode("414151344D57414D7977453D")))

    template = "FGWFDYYRozEqfKgzCAdI5GhQcIgTE4mgo8SJLKaxL6KK43ShsmyVGQtJgRhRuDUcRpwU5YUREQaCEuDSHH4gSMbkmGKMsjIWGQMlIzwtLn4dDxGCdPpOfiNWjcjrMnBdeEMYMAqhdlnEuNQyhd1W4GilFMWGEQ1CYhJwThhvDkdFIpgiXGosD5GCJJJMJRZZHKWN01DCPF48SZHEyhLJUWhfLLZOBCbqBGDhSRudBgA2KAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA=="
    feature = "FIWFFSSE4gAAFUieHAACTR26AABQg7hAAAHYrBgAAU+XYgAAInNnGAAAUntiAACMkSwgABNSfHgAAotTE4AARCsEsAAJ1WZiAAEertEAACWl2fAABTzKMQAAlVx6YAAC+7HQAAIyd5WAAFQvE6AAAiXmiAAAnxIAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAD0/wGQoFwCgABIAZAkAQCQAAAAAAcAAJAHAAKAAAAAAGDkAoD8rAKAAAAAAAAAAAAAAAAAAAAAAA=="
    print(bytes_to_hex_string(base64.b64decode(template)))
    print(bytes_to_hex_string(base64.b64decode(feature)))





#!/usr/bin/env python
# coding:utf-8
"""
# This is a wrapper module for module for pySerial by Liechti <cliechti@gmx.net>
#
# This file is part of pySerial. https://github.com/pyserial/pyserial
# by leo gong
#
"""
# 2017.09.17, gongleilei, pyserial 3.0之后的版本不兼容XP系统，增加对XP系统的兼容
import platform

if platform.platform().startswith("Windows"):
    os_maj_ver = platform.win32_ver()[1].split('.')[0]
    if int(os_maj_ver) >= 6: # windows vista and later
        import serial
        from serial.tools import list_ports
    else:
        # 兼容XP系统，在XP上只能运行pyserial<3.0
        from  .. import serial27 as serial
        from serial.tools import list_ports
else:
    import serial
    from serial.tools import list_ports


class SerialPortManager(object):
    serialdevices = []
    ports_info = {}

    @staticmethod
    def find_port(vid=None, pid=None):
        ports_info = []
        # pyserial=2.7， comports得到的是一个comports函数生成器对象(<generator object comports at ...>),
        # pyserial>3.0， comports得到的是一个列表（内部把得到的iterate_comports函数生成器对象转成了列表）
        comports = list_ports.comports()
        portinfolist = [SerialPortManager.__update_port_info(comport) for comport in comports]

        if vid is None and pid is None:
            ports_info = [(portinfo['device'], portinfo['description']) for portinfo in portinfolist]
        elif vid is not None and pid is None:
            for portinfo in portinfolist:
                if portinfo['vid'] == vid:
                    ports_info.append((portinfo['device'], portinfo['description']))
        elif vid is None and pid is not None:
            for portinfo in portinfolist:
                if portinfo['pid'] == pid:
                    ports_info.append((portinfo['device'], portinfo['description']))
        else:
            for portinfo in portinfolist:
                if portinfo['vid'] == vid and portinfo['pid'] == pid:
                    ports_info.append((portinfo['device'], portinfo['description']))

        return ports_info

    @staticmethod
    def __update_port_info(comport):
        """
        device ,name ,description, hwid
        #USB specific data
        vid ,pid ,serial_number,location ,manufacturer,product ,interface

        :param comport:
        :return:
        """
        # pyserial=2.7， comports得到的是一个comports函数生成器对象(<generator object comports at ...>),
        # yield 返回的数据格式(port, description, szHardwareID_str)
        # port: 串口名称
        # description: 友好名称
        # szHardwareID_str: 设备实例路径的描述
        # 对于USB串口，u'USB\\VID_10C4&PID_EA60\\0001'==> 'USB VID:PID=10C4:EA60 SNR=0001'
        # 对于其他串口不做处理，原样输出。'ACPI\\PNP0501\\1' ==> 'ACPI\\PNP0501\\1'
        #
        # pyserial>3.0， comports得到的是一个列表（内部把得到的iterate_comports函数生成器对象转成了列表）,
        # 列表元素是ListPortInfo的实例对象。
        # 字段device : unicode编码的串口名称
        # 字段description: 友好名称（从注册表中读取PnP设备的属性）
        # 以下为从设备实例路径szHardwareID_str（如，u'USB\\VID_10C4&PID_EA60\\0001'）中解析出的信息
        # 字段vid: 2字节16进制整数，0x10C4
        # 字段pid: 2字节16进制整数, 0xEA60
        # 字段serial_number: u'0001'
        # bInterfaceNumber: 无
        # 字段location: 由位置路径和bInterfaceNumber得出。
        # 如位置路径：1-1.4；PCIROOT(0)#PCI(1D00)#USBROOT(0)#USB(1)#USB(4)，bInterfaceNumber为空
        # 字段hwid: 对于USB,FTDIBUS和其他设备分别处理
        # 字段manufacturer: 厂商
        portinfodic = {}

        # pyserial = 2.7
        if isinstance(comport, tuple):
            portinfodic['device'] = comport[0]
            portinfodic['description'] = comport[1]
            portinfodic['hwid'] = comport[2]
            if comport[2].startswith('USB'):
                portinfodic['vid'] = int(comport[2].split("=")[1].split(':')[0][:4], 16)
                portinfodic['pid'] = int(comport[2].split("=")[1].split(':')[1][:4], 16)
            else:
                portinfodic['vid'] = None
                portinfodic['pid'] = None
        else:  # pyserial >= 3.0
            portinfodic['device'] = comport.device
            portinfodic['description'] = comport.description
            portinfodic['hwid'] = comport.hwid
            portinfodic['vid'] = comport.vid
            portinfodic['pid'] = comport.pid
        return portinfodic

    @staticmethod
    def open_port(portname, baudrate=115200, bytesize=8, timeout=None):
        try:
            serialport = serial.Serial()
            serialport.port = portname
            serialport.baudrate = baudrate
            serialport.stopbits = 1
            serialport.parity = 'N'
            serialport.bytesize = bytesize
            serialport.xonxoff = 0
            serialport.rtscts = 0
            serialport.timeout = timeout
            serialport.open()
            return serialport
        except serial.serialutil.SerialException as serialexp:
            errmesglist = str(serialexp).split(":")
            if len(errmesglist) == 2:
                obj = eval(errmesglist[1])
                if isinstance(obj, WindowsError):
                    winerrmsg = str(obj)#.decode('gbk').encode('utf8')
                    raise Exception("{}({})".format(errmesglist[0], winerrmsg))
            raise Exception(errmesglist[0])
        except Exception as e:
            raise e

    @staticmethod
    def close_port(serialport):
        if not isinstance(serialport, serial.Serial):
            return
        serialport.close()

    @staticmethod
    def send_to_port(serialport, bytestring):
        if not isinstance(serialport, serial.Serial):
            return
        serialport.write(bytestring)

    @staticmethod
    def read_from_port(serialport, size=1):
        if not isinstance(serialport, serial.Serial):
            return
        text = serialport.read(size)  # read one, with timout
        return text

    @staticmethod
    def read_set_timeout(serialport, timeout):
        """设置接收超时时间
        :param serialport:
        :param timeout:None-阻塞模式，其他情况：整数或浮点数
        :return:
        """
        serialport.timeout = timeout


if __name__ == "__main__":
    import ABCodeUtil
    from utilityhelper.loggingHelper import *  # @UnusedWildImport

    __fmt = '【%(levelname)s】 - %(asctime)s - %(name)s - %(process)d - %(thread)d -%(threadName)s - %(message)s'
    _g_logger = LogManger.getLogger(None, LEVEL_DEBUG)
    LogManger.addHandler((_g_logger,), HANDLER_TYPE_STREAM, LEVEL_DEBUG, __fmt)
    portlist = SerialPortManager.find_port()
    print("portlist=", portlist)
    serialport = SerialPortManager.open_port('COM5')
    _g_logger.debug("serialport=%s" % serialport)

    SerialPortManager.send_to_port(serialport, "\x01\x27\x00")
    _g_logger.debug("inwaiting=%d" % serialport.in_waiting)
    evt_op_len = SerialPortManager.read_from_port(serialport, 3)
    _g_logger.debug("event=%s" % ABCodeUtil.StringToHex(evt_op_len))
    if evt_op_len[0] == '\x02' and evt_op_len[1] == '\x06':
        _g_logger.debug("inwaiting=%d" % serialport.in_waiting)
        payload = SerialPortManager.read_from_port(serialport, ord(evt_op_len[2]))
        _g_logger.debug("pyload=%s" % ABCodeUtil.StringToHex(payload))
        if len(payload) >= 2:
            if payload[1] != '\x00':
                pass

    _g_logger.debug("connect.........")
    SerialPortManager.send_to_port(serialport, "\x01\x26\x06\x2F\xF7\x82\xE2\xB2\x4E")
    _g_logger.debug("inwaiting=%d" % serialport.in_waiting)
    evt_op_len = SerialPortManager.read_from_port(serialport, 3)
    _g_logger.debug("event=%s" % ABCodeUtil.StringToHex(evt_op_len))
    if evt_op_len[0] == '\x02' and evt_op_len[1] == '\x06':
        _g_logger.debug("inwaiting=%d" % serialport.in_waiting)
        payload = SerialPortManager.read_from_port(serialport, ord(evt_op_len[2]))
        _g_logger.debug("pyload=%s" % ABCodeUtil.StringToHex(payload))
        if len(payload) >= 2:
            if payload[1] != '\x00':
                pass
            else:
                _g_logger.debug("inwaiting=%d" % serialport.in_waiting)
                evt_op_len = SerialPortManager.read_from_port(serialport, 3)
                _g_logger.debug("pyload=%s" % ABCodeUtil.StringToHex(evt_op_len))

    _g_logger.debug("senddata.........")
    SerialPortManager.send_to_port(serialport, "\x01\x09\x0c\x2d\x00\x5a\x5b\x00\x00\x00\x00\x01\x02\x03\x04")
    _g_logger.debug("inwaiting=%d" % serialport.in_waiting)
    evt_op_len = SerialPortManager.read_from_port(serialport, 3)
    _g_logger.debug("event=%s" % ABCodeUtil.StringToHex(evt_op_len))
    if evt_op_len[0] == '\x02' and evt_op_len[1] == '\x06':
        _g_logger.debug("inwaiting=%d" % serialport.in_waiting)
        payload = SerialPortManager.read_from_port(serialport, ord(evt_op_len[2]))
        _g_logger.debug("pyload=%s" % ABCodeUtil.StringToHex(payload))
        if len(payload) >= 2:
            if payload[1] != '\x00':
                pass
            else:
                _g_logger.debug("inwaiting=%d" % serialport.in_waiting)
                evt_op_len = SerialPortManager.read_from_port(serialport, 3)
                _g_logger.debug("evt_op_len=%s" % ABCodeUtil.StringToHex(evt_op_len))
                if evt_op_len[0] == '\x02' and evt_op_len[1] == '\x08':
                    _g_logger.debug("inwaiting=%d" % serialport.in_waiting)
                    payload = SerialPortManager.read_from_port(serialport, ord(evt_op_len[2]))
                    _g_logger.debug("pyload=%s" % ABCodeUtil.StringToHex(payload))

    _g_logger.debug("disconnect.........")
    SerialPortManager.send_to_port(serialport, "\x01\x12\x00")
    _g_logger.debug("inwaiting=%d" % serialport.in_waiting)
    evt_op_len = SerialPortManager.read_from_port(serialport, 3)
    _g_logger.debug("event=%s" % ABCodeUtil.StringToHex(evt_op_len))
    if evt_op_len[0] == '\x02' and evt_op_len[1] == '\x06':
        _g_logger.debug("inwaiting=%d" % serialport.in_waiting)
        payload = SerialPortManager.read_from_port(serialport, ord(evt_op_len[2]))
        _g_logger.debug("pyload=%s" % ABCodeUtil.StringToHex(payload))
        if len(payload) >= 2:
            if payload[1] != '\x00':
                pass
            else:
                _g_logger.debug("inwaiting=%d" % serialport.in_waiting)
                evt_op_len = SerialPortManager.read_from_port(serialport, 3)
                _g_logger.debug("evt_op_len=%s" % ABCodeUtil.StringToHex(evt_op_len))



#coding:utf-8
import wx
import wx.grid
from utilityhelper.wxwrap import HelpGridTable


############################################################
class myGrid(wx.grid.Grid):
    def __init__(self, parent, dataTable, dataTypes = [], rowLabels=None, colLabels=None):
        wx.grid.Grid.__init__(self, parent, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, 0 )
        
        self.tableBase = HelpGridTable.GenericTable(dataTable, dataTypes, rowLabels, colLabels)
        
        self.SetTable(self.tableBase)
        
        #不显示行标题
        self.HideRowLabels()
        #self.SetRowLabelSize(0)
        
        #第一列设置为复选框
        self.SetColFormatBool(0) 
        
        #防止较宽列的数据重叠显示到附近的列
        self.SetDefaultCellOverflow(False)
        
        #行列宽度自适应 
        self.AutoSizeColumns(True)
        self.AutoSizeRows(True) 
        
        #表格单元格属性
        attr = wx.grid.GridCellAttr()
        attr.SetReadOnly(True)
        #除第一列外，表格不可编辑
        for i in range(1, len(dataTable[0])):
            self.SetColAttr(i, attr)  
        
        self.Bind(wx.grid.EVT_GRID_LABEL_LEFT_CLICK, self.OnLabelLeftClick)
        self.Bind(wx.grid.EVT_GRID_LABEL_LEFT_DCLICK, self.OnLabelLeftDClick)

        self.Bind(wx.grid.EVT_GRID_RANGE_SELECT, self.OnRangeSelect)
        self.Bind(wx.grid.EVT_GRID_SELECT_CELL, self.OnSelectCell)
         
    def OnLabelLeftClick(self, evt):
        print("OnCellLeftClick: (%d,%d) %s\n" %(evt.GetRow(), evt.GetCol(), evt.GetPosition()))        
        evt.Skip() 
 
    def OnLabelLeftDClick(self, evt):
        '''双击标题全选
        evt.GetRow(), 选中的行编号:当单击列标题时，返回行号为-1，列号从0开始
        evt.GetCol(), 选中的列编号:当单击行标题时，返回列号为-1，行号从0开始
        evt.GetPosition(),相对于网格左上角的坐标,表示为(x,y)
        '''
        print(("OnLabelLeftDClick: (%d,%d) %s\n" %
                       (evt.GetRow(), evt.GetCol(), evt.GetPosition())))
        
        allrows = self.GetNumberRows()
        for row in range(0, allrows):
            print("row(%d)=%s"%(row, self.GetCellValue(row, 0)))            
            if(self.GetCellValue(row, 0) == '1'):
                self.SetCellValue(row, 0 , '0')     
            else:
                self.SetCellValue(row, 0 , '1')         
        evt.Skip()

    def OnRangeSelect(self, evt):
        if evt.Selecting():
            top = (evt.GetTopLeftCoords().GetRow(),evt.GetTopLeftCoords().GetCol())
            bottom = (evt.GetBottomRightCoords().GetRow(),evt.GetBottomRightCoords().GetCol())

            for row in range(top[0], bottom[0]):
#                 booleditor = wx.grid.GridCellBoolEditor()
#                 self.SetCellEditor(row, top[1], booleditor)
                
#                 render = wx.grid.GridCellBoolRenderer()
#                 self.SetCellRenderer(row, top[1], render)
                
                pass 
#                 print "(%d,%d)=%s, %s", row, top[1], self.GetCellValue(row, top[1]), type(self.GetCellValue(row, top[1]))            
#                 print "int()=", int(self.GetCellValue(row, top[1]))
#                 print "==", self.GetCellEditor(row, top[1])                               
#                 if(self.GetCellValue(row, top[1]) != '' and
#                     int(self.GetCellValue(row, top[1])) == 0):
#                     self.SetCellValue(row, 0 , '1')
#                 else:
#                     self.SetCellValue(row, 0, '0')
                              
        else:
            pass

        evt.Skip()

    def OnSelectCell(self, evt):
        if evt.Selecting():
            msg = 'Selected'
        else:
            msg = 'Deselected'
        print(("OnSelectCell: %s (%d,%d) %s\n" %
                       (msg, evt.GetRow(), evt.GetCol(), evt.GetPosition())))

        # Another way to stay in a cell that has a bad value...
        row = self.GetGridCursorRow()
        col = self.GetGridCursorCol()

        if self.IsCellEditControlEnabled():
            self.HideCellEditControl()
            self.DisableCellEditControl()

        value = self.GetCellValue(row, col)

        evt.Skip()
        
        self.tableBase.DeleteRows(1, 2)
#         def DeleteRows(self, pos=3, numRows=1):
#             print "DeleteRows"

############################################################    
#使用这通用的表来显示阵容
colLabels = ('Select', 'Last', 'First', "gender ")
rowLabels = None
data = [[0, 'zhangsan', 26, 'man'],
        [1, 'lisi', 25, 'man']]
dataTypes = [wx.grid.GRID_VALUE_STRING,
             wx.grid.GRID_VALUE_STRING,
             wx.grid.GRID_VALUE_STRING]

class Frame(wx.Frame):
    def __init__(self, parent):
        wx.Frame.__init__(self, parent, -1, 'A Grid',
        size=(275, 275))
        grid = myGrid(self, data, dataTypes, rowLabels, colLabels)
        
if __name__ == '__main__':
    app = wx.App()
    frame = Frame(None)
    frame.Show(True)
    app.MainLoop()


#! python
#
# Python Serial Port Extension for Win32, Linux, BSD, Jython
# see __init__.py
#
# This module implements a special URL handler that uses the port listing to
# find ports by searching the string descriptions.
#
# (C) 2011 Chris Liechti <cliechti@gmx.net>
# this is distributed under a free software license, see license.txt
#
# URL format:    hwgrep://regexp

import serial27
import serial27.tools.list_ports

class Serial(serial27.Serial):
    """Just inherit the native Serial port implementation and patch the open function."""

    def setPort(self, value):
        """translate port name before storing it"""
        if isinstance(value, str) and value.startswith('hwgrep://'):
            serial27.Serial.setPort(self, self.fromURL(value))
        else:
            serial27.Serial.setPort(self, value)

    def fromURL(self, url):
        """extract host and port from an URL string"""
        if url.lower().startswith("hwgrep://"): url = url[9:]
        # use a for loop to get the 1st element from the generator
        for port, desc, hwid in serial27.tools.list_ports.grep(url):
            return port
        else:
            raise serial27.SerialException('no ports found matching regexp %r' % (url,))

    # override property
    port = property(serial27.Serial.getPort, setPort, doc="Port setting")

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
if __name__ == '__main__':
    #~ s = Serial('hwgrep://ttyS0')
    s = Serial(None)
    s.port = 'hwgrep://ttyS0'
    print(s)

